/* PhotoOrganizer 
 * Copyright (C) 1999 Dmitry Rogatkin.  All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 */
package photoorganizer;

import java.util.Hashtable;
import java.util.Enumeration;
import java.util.Properties;
import java.awt.event.*;
import java.awt.Component;
import java.awt.Image;
import java.awt.Toolkit;
import java.awt.FlowLayout;
import java.awt.PrintJob;
import java.awt.Color;
//import java.awt.print.PrinterJob;
import java.awt.Graphics;
import java.awt.Dimension;
import java.awt.image.ImageObserver;
import java.io.File;
import java.io.IOException;
import javax.swing.*;
import javax.swing.filechooser.FileFilter;

import photoorganizer.directory.*;
import photoorganizer.renderer.*;
import photoorganizer.formats.*;

public class Controller implements ActionListener, Persistentable {
    // component constraint
    public final static Integer COMP_IMAGEPANEL       = new Integer(1);
    public final static Integer COMP_DIRTREE          = new Integer(2);
    public final static Integer COMP_THUMBPANEL       = new Integer(3);
    public final static Integer COMP_FORMATS          = new Integer(4);
    public final static Integer COMP_COLLECTION       = new Integer(5);
    public final static Integer COMP_THUMBCOLLCTPANEL = new Integer(6);
    public final static Integer COMP_IMAGECOLLCTPANEL = new Integer(7);
    public final static Integer COMP_ALBUMPANEL       = new Integer(8);
    public final static Integer COMP_STATUSBAR        = new Integer(9);
    public final static Integer COMP_IMAGEALBUMPANEL  = new Integer(10);
    public final static Integer COMP_ALBUMTHUMBPANEL  = new Integer(11);

    public SampleJpeg sampleJpeg;

    private static final String WIN_ID = "Windows";
    private static final String WIN_PATH = "rundll32";
    private static final String WIN_FLAG = "url.dll,FileProtocolHandler";
    private static final String UNIX_PATH = "netscape";
    private static final String UNIX_FLAG = "-remote openURL";

    public Controller(PhotoOrganizer photoorganizer) {
        main = photoorganizer;
        components = new Hashtable();
        serializer = new Serializer();
        sampleJpeg = new SampleJpeg();
	filter     = new HTMLFilter();
	mainicon   = Toolkit.getDefaultToolkit().getImage(ClassLoader.getSystemResource("photoorganizer.jpg"));
        serializer.load();
    }   
    
    public void actionPerformed(ActionEvent a) {
        String cmd = a.getActionCommand();
        if (cmd.equals(Resources.MENU_OPTIONS)) {
            if (options == null) {
                options = new OptionsFrame(this);
            } else 
                options.setVisible(true);
	} else if (cmd.equals(Resources.MENU_ABOUT)) {
	    JOptionPane.showMessageDialog(main, "<html><i>"+PhotoOrganizer.PROGRAMNAME+"\n"+
		PhotoOrganizer.VERSION+'\n'+
		PhotoOrganizer.COPYRIGHT+'\n'+
		"Java "+System.getProperty("java.version")+'\n'+
		"JVM "+System.getProperty("java.vendor")+
		" OS "+System.getProperty("os.name"),
		Resources.MENU_ABOUT, JOptionPane.PLAIN_MESSAGE,
		new ImageIcon(ClassLoader.getSystemResource("logo.jpg")));
	} else if (cmd.equals(Resources.MENU_CONTENTS)) {
            showUrl(Resources.URL_HELP);
	} else if (cmd.equals(Resources.MENU_VIEW_HTML)) {
            String d = (String)serializer.getProperty(WebPublishOptionsTab.SECNAME, WebPublishOptionsTab.WEBROOT);
            if (d == null || d.length() == 0)
                d = serializer.getHomeDirectory();
            JFileChooser fc = new JFileChooser(d);
            fc.addChoosableFileFilter(filter);
            fc.setFileFilter(filter);
            fc.setDialogTitle(Resources.TITLE_HTML_TEMPL);
            if (fc.showOpenDialog(main) == JFileChooser.APPROVE_OPTION)
                showUrl(fc.getSelectedFile().getAbsolutePath());
	} else if (cmd.equals(Resources.MENU_EXIT)) {
            close();
            main.dispose();
            System.exit(0);
        }
    }

    public boolean showUrl(String url) {
        String cmd = "";
        try {
            if (isWindowsPlatform()) {
                // cmd = 'rundll32 url.dll,FileProtocolHandler http://...'
                cmd = WIN_PATH + " " + WIN_FLAG + " " + url;
                Process p = Runtime.getRuntime().exec(cmd);
            } else {
                // Under Unix, Netscape has to be running for the "-remote"
                // command to work.  So, we try sending the command and
                // check for an exit value.  If the exit command is 0,
                // it worked, otherwise we need to start the browser.

                // cmd = 'netscape -remote openURL(http://www.javaworld.com)'
                cmd = UNIX_PATH + " " + UNIX_FLAG + "(" + url + ")";
                Process p = Runtime.getRuntime().exec(cmd);

                try {
                    // wait for exit code -- if it's 0, command worked,
                    // otherwise we need to start the browser up.
                    int exitCode = p.waitFor();

                    if (exitCode != 0){
                        // Command failed, start up the browser
                        // cmd = 'netscape http://www.javaworld.com'
                        cmd = UNIX_PATH + " "  + url;
                        p = Runtime.getRuntime().exec(cmd);
                    }
                } catch(InterruptedException x) {
                    System.err.println("Error bringing up browser, cmd='" + cmd + "'\nCaught: " + x);
                    return false;
                }
            }
        } catch(IOException x) {
            // couldn't exec browser
            System.err.println("Could not invoke browser, using "+cmd+"\n"+x);
            return false;
        }
        return true;
    }

    public synchronized void print(File[] files) {
        final PrintJob pj = Toolkit.getDefaultToolkit().getPrintJob(getMainFrame(),
            Resources.TITLE_PRINT, new Properties());
        final StatusBar statusbar = (StatusBar)component(COMP_STATUSBAR);
        statusbar.displayInfo(Resources.INFO_PRINTING);
        if (files != null)
            statusbar.setProgress(files.length*2);
        final Graphics g = pj.getGraphics();
        g.setColor(Color.white);
        final Dimension pd = pj.getPageDimension();
        for (int i=0; files != null && i<files.length; i++) {
            g.setClip(0,i*pd.height, pd.width, pd.height);
            g.drawRect(0,20, pd.width-10, pd.height-20-10);
            Image im = new BasicJpeg(files[i]).getImage();
            final boolean lastimage = i == (files.length-1);
            if (im == null)
                continue;
            final File file = files[i];
            if (!g.drawImage(im, 0,0,
                new ImageObserver() {
                public boolean imageUpdate(Image img, int infoflags, int x, int y,
                    int width, int height) {
                    if (y == height/2 && height != 0)
                        statusbar.tickProgress();
                    if ((infoflags & (ImageObserver.ALLBITS | ImageObserver.ERROR)) == 0)
                        return true;
                    else {
                        g.drawImage(img, 0,15, pd.width-30, (pd.width-30)*height/width, null);
			img.flush();
                        g.drawString(file.getAbsolutePath(), 15, (pd.width-30)*height/width+30);
                        statusbar.tickProgress();
                        if (lastimage) {
                            g.dispose();
                            pj.end();
                            statusbar.clearInfo();
                            statusbar.clearProgress();
                        }
                        return false;
                    }
                }
            }
            ))
                ;
        }
    }

    public static boolean isWindowsPlatform()
    {
        String os = System.getProperty("os.name");
        if ( os != null && os.startsWith(WIN_ID))
            return true;
        else
            return false;
    }

    public void add(Component component, Object constraint) {
        components.put(constraint, component);
    }

    public Component component(Object constraint) {
        return (Component)components.get(constraint);
    }

    public void fireDisplayImage(BasicJpeg format) {
        if (format == null || !format.isValid())
            return;
        ((PhotoImagePanel)component(COMP_IMAGEPANEL)).updateImage(format);
        updateCaption(format.getLocationName());
    }
    
    public void updateCaption(String str) {
        if (str == null)
            str = "";
        if (str.length() > 0)
            str += " - ";
        main.setTitle(str+PhotoOrganizer.PROGRAMNAME+" "+PhotoOrganizer.VERSION);
    }

    public JFrame getMainFrame() {
        return main;
    }

    public Image getMainIcon() {
        return mainicon;
    }

    public void save() {
        Enumeration ee = components.elements();
        Object o;
        while(ee.hasMoreElements()) {
            o = ee.nextElement();
            if (o instanceof Persistentable)
                ((Persistentable)o).save();
        }
    }
    
    public void load() {
        Enumeration ee = components.elements();
        Object o;
        while(ee.hasMoreElements()) {
            o = ee.nextElement();
            if (o instanceof Persistentable)
                ((Persistentable)o).load();
        }
    }
    
    public Serializer getSerializer() {
        return serializer;
    }

    public FileFilter getHtmlFilter() {
	return filter;
    }

    public void close() {
        save();
        main.save();
        serializer.save();
    }

    public static JPanel createButtonPanel(ActionListener al) {
        JButton btn;
        JPanel result = new JPanel();
        result.setLayout(new FlowLayout(FlowLayout.RIGHT));
        result.add(btn = new JButton(Resources.CMD_OK));
        btn.addActionListener(al);
        result.add(btn = new JButton(Resources.CMD_APPLY));
        btn.addActionListener(al);
        result.add(btn = new JButton(Resources.CMD_CANCEL));
        btn.addActionListener(al);
        return result;
    }

    public static JMenu createTransformMenu(ActionListener listener) {
	JMenu result = new JMenu(Resources.MENU_TRANSFORM);
	JMenuItem item;
	result.add(item = new JMenuItem(Resources.MENU_NONE));
	item.addActionListener(listener);
	result.add(item = new JMenuItem(Resources.MENU_ROTATE90));
	item.addActionListener(listener);
	result.add(item = new JMenuItem(Resources.MENU_ROTATE180));
	item.addActionListener(listener);
	result.add(item = new JMenuItem(Resources.MENU_ROTATE270));
	item.addActionListener(listener);
	result.add(item = new JMenuItem(Resources.MENU_FLIP_H));
	item.addActionListener(listener);
	result.add(item = new JMenuItem(Resources.MENU_FLIP_V));
	item.addActionListener(listener);
	result.add(item = new JMenuItem(Resources.MENU_TRANSPOSE));
	item.addActionListener(listener);
	result.add(item = new JMenuItem(Resources.MENU_TRANSVERSE));
	item.addActionListener(listener);
	return result;
    }
    
    public static int convertCmdToTrnasformOp(String cmd) {
	int op = -1;
        if (cmd.equals(Resources.MENU_ROTATE90)) {
	    op = BasicJpeg.ROT_90;
        } else if (cmd.equals(Resources.MENU_ROTATE180)) {
	    op = BasicJpeg.ROT_180;
        } else if (cmd.equals(Resources.MENU_ROTATE270)) {
	    op = BasicJpeg.ROT_270;
        } else if (cmd.equals(Resources.MENU_FLIP_H)) {
	    op = BasicJpeg.FLIP_H;
        } else if (cmd.equals(Resources.MENU_FLIP_V)) {
	    op = BasicJpeg.FLIP_V;
        } else if (cmd.equals(Resources.MENU_TRANSPOSE)) {
	    op = BasicJpeg.TRANSPOSE;
        } else if (cmd.equals(Resources.MENU_TRANSVERSE)) {
	    op = BasicJpeg.TRANSVERSE;
        } else if (cmd.equals(Resources.MENU_NONE)) {
	    op = BasicJpeg.NONE;
        }
        return op;
    }

    class HTMLFilter extends FileFilter  {
        final String [] extensions = new String[] {"html", "htm", "htmt"};
        
        // Accept all directories and (gif || jpg || tiff) files.
        public boolean accept(File f) {
            if(f.isDirectory()) {
                return true;
            }
            String s = f.getName();
            int i = s.lastIndexOf('.');
            if(i > 0 &&  i < s.length() - 1) {
                String extension = s.substring(i+1).toLowerCase();
                for (i=0; i<extensions.length; i++) {
                    if (extensions[i].equals(extension)) {
                        return true;
                    } else {
                        return false;
                    }
                }
            }
            return false;
        }
        
        // The desctiption of this filter
        public String getDescription() {
            return Resources.LABEL_HTML_FILES;
        }
    }

    private Hashtable components;
    private PhotoOrganizer main;
    private Image mainicon;
    private String transformtargetdir;
    private OptionsFrame options;
    private Serializer serializer;
    private HTMLFilter filter;

    /*
        synchronized(pj) {
            pj.notify();
        }
        synchronized(pj) {
            
            try {
                pj.wait(10*60*1000);
            } catch(InterruptedException e) {
            }
        }*/

}

