/* PhotoOrganizer 
* Copyright (C) 1999 Dmitry Rogatkin.  All rights reserved.
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions
* are met:
* 1. Redistributions of source code must retain the above copyright
*    notice, this list of conditions and the following disclaimer.
* 2. Redistributions in binary form must reproduce the above copyright
*    notice, this list of conditions and the following disclaimer in the
*    documentation and/or other materials provided with the distribution.
*/
package photoorganizer;
import java.io.*;
import java.net.*;
import java.awt.Image;
import javax.swing.JOptionPane;
import photoorganizer.renderer.*;
import photoorganizer.formats.*;
import photoorganizer.ftp.*;

/** The class parses HTML template and it's looking for special tags
!%loop - start of a loop
!%tnof - substitute original file name
!%tnfn - substitute thumbnail file name
!%tntt - tooltip for thumbnail
!%tnc  - commentary under
!%tnw  - width of thumbnail image (optional)
!%tnh  - height of thumbnail image (optional)
!%tn+  - advance to next thumbnail
!%endl - end of the loop
*/
public class HtmlProducer {
    
    private final int HTML           = 1;
    private final int PERCENT        = 2;
    private final int EXCLAM         = 3;
    private final int L              = 4;
    private final int O              = 5;
    private final int O2             = 6;
    private final int P              = 7;
    private final int L_HTML         = 8;
    private final int E              = 9;
    private final int N              = 10;
    private final int D              = 11;
    private final int L_PERCENT      = 12;
    private final int L_EXCLAM       = 13;
    private final int T              = 14;
    private final int F              = 15;
    private final int TNT            = 16;
    private final int SKIPOUT        = 17;
    private final int SK_EXCLAM      = 18;
    private final int SK_PERCENT     = 19;
    private final int SK_T           = 20;
    private final int SK_N           = 21;
    
    final static char CPERCENT = '%';
    final static char CEXLAM = '!';
    public HtmlProducer(Controller controller) {
	this.controller = controller;
    }
    
    public void produce(String desthtmlname, File[] files) throws IOException {
	Serializer s = controller.getSerializer();
	BufferedInputStream is = new BufferedInputStream(new FileInputStream(new File(
	    (String)s.getProperty(WebPublishOptionsTab.SECNAME, WebPublishOptionsTab.HTMLTEMPL))));
	OutputStream os;
        FtpCommandReply fcr = null;
        Integer i = (Integer)s.getProperty(WebPublishOptionsTab.SECNAME, WebPublishOptionsTab.USEFTP);
        if (i != null && i.intValue() == 1) {
            Ftp ftp = new Ftp(WebPublishOptionsTab.getConnectionInfo(s));
	    if (!ftp.isConnected) {
                JOptionPane.showMessageDialog(controller.getMainFrame(), Resources.LABEL_ERR_FTP_CONNECT,
                    Resources.TITLE_ERROR, JOptionPane.ERROR_MESSAGE);
                return ;
            }
            try {
                ftp.login();
                ftp.cwd();
                fcr = ftp.stor(desthtmlname);
                if(fcr.serverSocket != null || fcr.socket == null)
                    fcr.socket = fcr.serverSocket.accept();
                os = fcr.socket.getOutputStream();
            } catch (InterruptedIOException e) {
                if(fcr.serverSocket != null)
                    fcr.serverSocket.close();
                return;
            } catch (IOException e) {
                if(fcr.serverSocket != null)
                    fcr.serverSocket.close();
                JOptionPane.showMessageDialog(controller.getMainFrame(), Resources.LABEL_ERR_FTP_LOGIN+e,
                    Resources.TITLE_ERROR, JOptionPane.ERROR_MESSAGE);
                return ;
            }
        } else {
            os = new FileOutputStream(new File(
                (String)s.getProperty(WebPublishOptionsTab.SECNAME, WebPublishOptionsTab.WEBROOT), desthtmlname));
        }
	
        StringBuffer sb = new StringBuffer();
	int status = HTML;
	char c;
	int ic;
        while((ic = is.read()) >=0)
        {
            c = (char)(ic & 255);
	    switch(status)
            {
            case HTML: 
		sb.setLength(0);
                if(c == CEXLAM) {
                    status=EXCLAM;
		    sb.append(c);
		} else
		    os.write(c);
                break;
            case EXCLAM: 
		sb.append(c);
                if(c==CPERCENT)
                    status=PERCENT;
                else {
                    status=HTML;
		    write(os, sb);
		}
                break;
            case PERCENT: 
                sb.append(c);
		if(c=='l' || c=='L')
                    status=L; // we excpect only loop, altough it can not be
                else {
                    status=HTML;
		    write(os, sb);
		}
                break;
            case L: 
                sb.append(c);
                if(c=='o' || c=='O')
                    status=O;
                else {
                    status=HTML;
		    write(os, sb);
		}
                break;
            case O: 
                sb.append(c);
                if(c=='o' || c=='O')
                    status=O2;
                else {
                    status=HTML;
		    write(os, sb);
		}
                break;
            case O2: 
                sb.append(c);
                if(c=='p' || c=='P') {
                    status=L_HTML;
		    sb.setLength(0);
                } else {
                    status=HTML;
		    write(os, sb);
		}
                break;
            case L_HTML: 
                sb.append(c);
                if(c==CEXLAM)
                    status=L_EXCLAM;
                else
                    status=L_HTML;
                break;
            case L_EXCLAM: 
		sb.append(c);
                if(c==CPERCENT)
                    status=L_PERCENT;
                else 
                    status=L_HTML;
                break;
            case L_PERCENT: 
		sb.append(c);
                if(c=='e' || c=='E')
                    status=E;
                else
                    status=L_HTML;
                break;
            case E: 
		sb.append(c);
                if(c=='n' || c=='N')
                    status=N;
                else
                    status=L_HTML;
                break;
            case N: 
		sb.append(c);
                if(c=='d' || c=='D')
                    status=D;
                else
                    status=L_HTML;
                break;
            case D: 
		sb.append(c);
                if(c=='l' || c=='L') {
                    status=HTML;
		    sb.setLength(sb.length()-6);
		    processLoop(os, sb, files, i != null && i.intValue() == 1);
                } else
                    status=L_HTML;
                break;
	    }
	}            
	if(status != HTML)
	{
	    write(os, sb);
	} 
	os.flush();
	os.close();
        if (fcr != null) {
            if(fcr.socket != null)
                fcr.socket.close();
            if(fcr.serverSocket != null)
                fcr.serverSocket.close();
        }
	is.close();
    }

    void processLoop(OutputStream os, StringBuffer lbuf, File[] files, boolean useftp) throws IOException {
        StringBuffer sb = new StringBuffer();
        Serializer s = controller.getSerializer();
        String destpath = (String)s.getProperty(WebPublishOptionsTab.SECNAME, WebPublishOptionsTab.WEBROOT);
        if (destpath == null || destpath.length() == 0)
            destpath = controller.getSerializer().getHomeDirectory();
        for (int tn=0; tn<files.length;) {
            int status = HTML;
            char c;
	    int ln = tn;
            BasicJpeg format = new BasicJpeg(files[tn]);
            for(int i=0; i<lbuf.length(); i++)
            {
                c = lbuf.charAt(i);
                switch(status)
                {
                case HTML: 
                    sb.setLength(0);
                    if(c == CEXLAM) {
                        status=EXCLAM;
                        sb.append(c);
                    } else
                        os.write(c);
                    break;
                case EXCLAM: 
                    sb.append(c);
                    if(c==CPERCENT)
                        status=PERCENT;
                    else {
                        status=HTML;
                        write(os, sb);
                    }
                    break;
                case PERCENT: 
                    sb.append(c);
                    if(c=='t' || c=='T')
                        status=T;
                    else {
                        status=HTML;
                        write(os, sb);
                    }
                    break;
                case T: 
                    sb.append(c);
                    if(c=='n' || c=='N')
                        status=N;
                    else {
                        status=HTML;
                        write(os, sb);
                    }
                    break;
                case N: 
                    sb.append(c);
                    if(c=='o' || c=='O')
                        status=O;
                    else if(c=='f' || c=='F')
                        status=F;
                    else if(c=='t' || c=='T')
                        status=TNT;
                    else if(c=='c' || c=='C') {
                        status=HTML;
                        sb.setLength(0);
                        String l = (String)s.getProperty(ThumbnailsOptionsTab.SECNAME, ThumbnailsOptionsTab.LABELMASK);
                        if (l != null)
                            sb.append(new FileNameFormat(l).format(format));
                        write(os, sb);
                    } else if(c=='w' || c=='W') {
                        status=HTML;
                        sb.setLength(0);
                        AbstractImageInfo ii = format.getImageInfo();
                        if (ii != null) {
                            Image im = ii.getThumbnailImage(format);
                            if (im != null)
                                sb.append(im.getWidth(null));
                        }
                        write(os, sb);
                    } else if(c=='h' || c=='H') {
                        status=HTML;
                        sb.setLength(0);
                        AbstractImageInfo ii = format.getImageInfo();
                        if (ii != null) {
                            Image im = ii.getThumbnailImage(format);
                            if (im != null)
                                sb.append(im.getHeight(null));
                        }
                        write(os, sb);
                    } else if(c=='+') {
                        sb.setLength(0);
                        tn++;
                        if (tn>=files.length) {
                            // skip to last !%tn+
			    status=SKIPOUT;
                        } else {
                            format = new BasicJpeg(files[tn]);
                            status=HTML;
                        }
                    } else {
                        status=HTML;
                        write(os, sb);
                    }
                    break;
                case TNT: 
                    sb.append(c);
                    status=HTML;
                    if(c=='t' || c=='T') {
                        status=HTML;
                        sb.setLength(0);
                        String l = s.arrayToString(s.getProperty(ThumbnailsOptionsTab.SECNAME, ThumbnailsOptionsTab.TOOLTIPMASK));
                        if (l != null)
                            sb.append(new FileNameFormat(l).format(format));
                    }
                    write(os, sb);
                    break;
                case F: 
                    sb.append(c);
                    status=HTML;
                    if(c=='n' || c=='N') {
                        sb.setLength(0);
                        String m = (String)s.getProperty(ThumbnailsOptionsTab.SECNAME, ThumbnailsOptionsTab.FILEMASK);
                        if (m == null || m.length() == 0)
                            m = PhotoCollectionPanel.DEFTNMASK;
                        AbstractImageInfo ii = format.getImageInfo();
                        try {
                            if (ii != null) {
                                sb.append(FileNameFormat.makeValidPathName(new FileNameFormat(m).format(format)));
                                OutputStream tnos;
                                FtpCommandReply fcr = null;
                                if (useftp) {
                                    Ftp ftp = new Ftp(WebPublishOptionsTab.getConnectionInfo(controller.getSerializer()));
				    if (!ftp.isConnected) {
					System.err.println(Resources.LABEL_ERR_FTP_CONNECT);
					break ;
				    }
                                    ftp.login();
                                    ftp.cwd();
                                    fcr = ftp.stor(sb.toString());
                                    try {
                                        if(fcr.serverSocket != null || fcr.socket == null)
                                            fcr.socket = fcr.serverSocket.accept();
                                    } catch (InterruptedIOException ie) {
                                        if(fcr.serverSocket != null)
                                            fcr.serverSocket.close();
                                        System.err.println(""+ie);
                                        break;
                                    }
                                    tnos = fcr.socket.getOutputStream();
                                } else
                                    tnos = new FileOutputStream(new File(destpath, sb.toString()));
                                ii.saveThumbnailImage(format, tnos);
                                if (useftp) {
                                    if(fcr.socket != null)
                                        fcr.socket.close();
                                    if(fcr.serverSocket != null)
                                        fcr.serverSocket.close();
                                }
                            }
                        } catch(IOException ioe) {
                            System.err.println(""+ioe);
                        }
                    }
                    write(os, sb);
                    break;
                case O: 
                    sb.append(c);
                    status=HTML;
                    if(c=='f' || c=='F') {
                        status=HTML;
                        sb.setLength(0);
                        sb.append(format.getLocationName());
                    }
                    write(os, sb);
                    break;
		case SKIPOUT:
		    sb.append(c);
                    if(c == CEXLAM)
                        status=SK_EXCLAM;
		    break;
		case SK_EXCLAM:
		    sb.append(c);
                    if(c==CPERCENT)
                        status=SK_PERCENT;
		    break;
		case SK_PERCENT:
		    sb.append(c);
                    if(c=='t' || c=='T') 
			status=SK_T;
		    break;
		case SK_T:
		    sb.append(c);
                    if(c=='n' || c=='N') 
			status=SK_N;
		    break;
		case SK_N:
		    sb.append(c);
                    if(c=='+') {
			status=SKIPOUT;
			sb.setLength(0);
		    }
		    break;
                }
            }            
            if(status != HTML)
            {
                write(os, sb);
            }
	    if (ln==tn)
		tn++;
        }
    }

    void write(OutputStream os, StringBuffer sb) throws IOException {
	for (int i=0; i<sb.length(); i++)
	    os.write(sb.charAt(i));
    }

    Controller controller;
}