/* PhotoOrganizer 
 * Copyright (C) 1999 Dmitry Rogatkin.  All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 */
package photoorganizer;

import javax.swing.*;
import javax.swing.border.*;
import java.awt.event.*;
import java.awt.Rectangle;
import java.awt.Dimension;
import java.awt.BorderLayout;
import javax.swing.event.MouseInputAdapter;
import java.io.*;
import photoorganizer.directory.*;
import photoorganizer.renderer.*;
import photoorganizer.formats.FileNameFormat;

public class PhotoOrganizer extends JFrame implements Persistentable {

    public static final String PROGRAMNAME = "R-PhotoOrganizer";
    public static final String VERSION = "version 1.0 beta2";
    public static final String COPYRIGHT = "Copyright (C) 1999 Dmitry Rogatkin";
    
    static final String SECNAME  = "PhotoOrganizer";
    static final String BOUNDS   = "Bounds";
    static final String DIVIDERDIR  = "DividerBrowsePos";
    static final String DIVIDERCOL  = "DividerCollectionPos";
    static final String DIVIDERALBUM  = "DividerAlbumPos";

    public static void main(String[] args) {
        new PhotoOrganizer();
    }
    
    public PhotoOrganizer() {
	try {
	    System.setErr(new PrintStream(new FileOutputStream(PROGRAMNAME+".log"), true));
	} catch (IOException e)
	{
	    System.err.println(PROGRAMNAME+": Can't redefine error stream.");
	}
	FlashWindow fw = new FlashWindow();
	// pros
        // can expand component to access to new component without changing constructors
        // or adding new methods
        // cons
        // longer access to other component, 
	// class cast isn't relaible 
        // order of creation can be important, if we access controller in constructor
        // to get an access to other components
        controller = new Controller(this);
        controller.updateCaption(null);
	setIconImage(controller.getMainIcon());
        addWindowListener( new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                controller.close();
                super.windowClosing(e);
            }
        });

        controller.add(new StatusBar(), Controller.COMP_STATUSBAR);
        controller.add(new PhotoImagePanel(controller), Controller.COMP_IMAGEPANEL);
        controller.add(new PhotoImagePanel(controller), Controller.COMP_IMAGECOLLCTPANEL);
        controller.add(new PhotoImagePanel(controller), Controller.COMP_IMAGEALBUMPANEL);
        controller.add(new CollectionThumbnailsPanel(controller), Controller.COMP_THUMBCOLLCTPANEL);
        controller.add(new PhotoCollectionPanel(controller), Controller.COMP_COLLECTION);
        controller.add(new AlbumPane(controller), Controller.COMP_ALBUMPANEL);
        controller.add(new ThumbnailsPanel(controller), Controller.COMP_THUMBPANEL);
        controller.add(new JTreeTable(new FileSystemModel(), controller), Controller.COMP_DIRTREE);
        controller.add(new AlbumThumbnailsPanel(controller), Controller.COMP_ALBUMTHUMBPANEL);

        JTabbedPane uptabbedpane = new JTabbedPane(SwingConstants.LEFT);
        uptabbedpane.insertTab(Resources.TAB_DIRECTORY, (Icon)null,
                new JScrollPane(controller.component(Controller.COMP_DIRTREE)),
                Resources.TTIP_DIRECTORYTAB, 0);
        uptabbedpane.insertTab(Resources.TAB_IMAGE, (Icon)null,
                controller.component(Controller.COMP_IMAGEPANEL),
                Resources.TTIP_IMAGETAB, 1);
        JScrollPane botmpane = new JScrollPane(controller.component(Controller.COMP_THUMBPANEL));

        Serializer s = controller.getSerializer();
	JTabbedPane gentabpane = new JTabbedPane(Serializer.getInt(
	    s.getProperty(MiscellaneousOptionsTab.SECNAME, MiscellaneousOptionsTab.TABPOS), SwingConstants.BOTTOM));
        gentabpane.insertTab(Resources.TAB_BROWSE, (Icon)null,
                new JSplitPane(JSplitPane.VERTICAL_SPLIT,
                true, uptabbedpane, botmpane), Resources.TTIP_BROWSETAB, 0);
        
        uptabbedpane = new JTabbedPane(SwingConstants.LEFT);
        uptabbedpane.insertTab(Resources.TAB_COLLECTION, (Icon)null,
                new JScrollPane(controller.component(Controller.COMP_COLLECTION)),
                Resources.TTIP_COLLECTTAB, 0);
        uptabbedpane.insertTab(Resources.TAB_IMAGE, (Icon)null,
                controller.component(Controller.COMP_IMAGECOLLCTPANEL),
                Resources.TTIP_IMAGETAB, 1);
        
        botmpane = new JScrollPane(controller.component(Controller.COMP_THUMBCOLLCTPANEL));

        gentabpane.insertTab(Resources.TAB_COLLECTION, (Icon)null,
                new JSplitPane(JSplitPane.VERTICAL_SPLIT,
                true, uptabbedpane, botmpane), Resources.TTIP_COLLECTLIST, 1);

        botmpane = new JScrollPane(controller.component(Controller.COMP_ALBUMTHUMBPANEL));
        uptabbedpane = new JTabbedPane(SwingConstants.LEFT);
        uptabbedpane.insertTab(Resources.TAB_ALBUM, (Icon)null,
                new JScrollPane(controller.component(Controller.COMP_ALBUMPANEL)),
                Resources.TTIP_COLLECTTAB, 0);
        uptabbedpane.insertTab(Resources.TAB_IMAGE, (Icon)null,
                controller.component(Controller.COMP_IMAGEALBUMPANEL),
                Resources.TTIP_IMAGETAB, 1);

        gentabpane.insertTab(Resources.TAB_ALBUM, (Icon)null,
            new JSplitPane(JSplitPane.VERTICAL_SPLIT,
            true, uptabbedpane, botmpane), Resources.TTIP_ALBUM, 2);

        getContentPane().add(gentabpane, BorderLayout.CENTER);
	
	addWindowListener(new WindowAdapter() {
	    public void windowClosing(WindowEvent we) {
		System.exit(0);
	    }
	});
        pack();
	load();
        setVisible(true);
        controller.load();
	fw.dispose();
    }

    public void save() {
	Serializer s = controller.getSerializer();
	Rectangle d = getBounds();
	Integer[] b = new Integer[4];
	b[0] = new Integer(d.x);
	b[1] = new Integer(d.y);
	b[2] = new Integer(d.width);
	b[3] = new Integer(d.height);
	s.setProperty(SECNAME, BOUNDS, b);
	JSplitPane sp = (JSplitPane)((JTabbedPane)getContentPane().getComponent(0)).getComponentAt(0);
	s.setProperty(SECNAME, DIVIDERDIR, new Integer(sp.getDividerLocation()));
	sp = (JSplitPane)((JTabbedPane)getContentPane().getComponent(0)).getComponentAt(1);
	s.setProperty(SECNAME, DIVIDERCOL, new Integer(sp.getDividerLocation()));
	sp = (JSplitPane)((JTabbedPane)getContentPane().getComponent(0)).getComponentAt(2);
	s.setProperty(SECNAME, DIVIDERALBUM, new Integer(sp.getDividerLocation()));
    }

    public void load() {
	Serializer s = controller.getSerializer();
	Object[] b = (Object[])s.getProperty(SECNAME, BOUNDS);
	if (b != null && b.length == 4 && b[0] instanceof Integer) {
	    setBounds(((Integer)b[0]).intValue(), ((Integer)b[1]).intValue(),
                ((Integer)b[2]).intValue(), ((Integer)b[3]).intValue());
	}
        JSplitPane sp = (JSplitPane)((JTabbedPane)getContentPane().getComponent(0)).getComponentAt(0);
	Integer l = (Integer)s.getProperty(SECNAME, DIVIDERDIR);
        if (l != null) {
            sp = (JSplitPane)((JTabbedPane)getContentPane().getComponent(0)).getComponentAt(0);
	    sp.setDividerLocation(l.intValue());
        }
        l = (Integer)s.getProperty(SECNAME, DIVIDERCOL);
        if (l != null) {
            sp = (JSplitPane)((JTabbedPane)getContentPane().getComponent(0)).getComponentAt(1);
	    sp.setDividerLocation(l.intValue());
        }
        l = (Integer)s.getProperty(SECNAME, DIVIDERALBUM);
        if (l != null) {
            sp = (JSplitPane)((JTabbedPane)getContentPane().getComponent(0)).getComponentAt(2);
	    sp.setDividerLocation(l.intValue());
        }
	if (Serializer.getInt(s.getProperty(MiscellaneousOptionsTab.SECNAME, MiscellaneousOptionsTab.MENUBAR), 1) == 1)
	    setJMenuBar(createMenu());
	else {
	    getContentPane().getComponent(0).addMouseListener(new MouseInputAdapter() {
		public void mouseClicked(MouseEvent e) {
		    if ((e.getModifiers() & InputEvent.BUTTON3_MASK) > 0)
			new ROptionsMenu().show(PhotoOrganizer.this, e.getX(), e.getY());
		}
	    });
	}
	if (Serializer.getInt(s.getProperty(MiscellaneousOptionsTab.SECNAME, MiscellaneousOptionsTab.TOOLBAR), 0) == 1)
	    getContentPane().add(createToolBar(), BorderLayout.NORTH);
	if (Serializer.getInt(s.getProperty(MiscellaneousOptionsTab.SECNAME, MiscellaneousOptionsTab.STATUSBAR), 1) == 1) {
	    getContentPane().add(controller.component(Controller.COMP_STATUSBAR), BorderLayout.SOUTH);
	}
        // load FileNameFormat data
	FileNameFormat.datemask = s.arrayToString(s.getProperty(MiscellaneousOptionsTab.SECNAME, MiscellaneousOptionsTab.DATEFORMAT));
	FileNameFormat.timemask = s.arrayToString(s.getProperty(MiscellaneousOptionsTab.SECNAME, MiscellaneousOptionsTab.TIMEFORMAT));
        FileNameFormat.counter = Serializer.getInt(s.getProperty(RenameOptionsTab.SECNAME, RenameOptionsTab.COUNTER), 0);
        FileNameFormat.transformCodes = new Object[Resources.ROTATIONS.length];
        Object []os;
        FileNameFormat.transformCodes = new Object[Resources.ROTATIONS.length];
	try {
            os = (Object[])s.getProperty(TransformOptionsTab.SECNAME, TransformOptionsTab.TRANSFORM);
	    if (os == null)
		throw new Exception("Transform option's missed in configuration file");
	} catch(Exception e) {
	    os = Resources.ROTATIONS;
	}
        System.arraycopy(os, 0, FileNameFormat.transformCodes, 0, os.length);
    }
    
    JMenuBar createMenu() {
        JMenuBar menubar = new JMenuBar();
        JMenu menu;
        JMenuItem item;
        menubar.add(menu = new JMenu(Resources.MENU_FILE));
        menu.add(item = new JMenuItem(Resources.MENU_DRIVE_SEL));
        item.addActionListener((JTreeTable)controller.component(Controller.COMP_DIRTREE));
        menu.add(item = new JMenuItem(Resources.MENU_ADDTOCOLLECT));
        item.addActionListener((JTreeTable)controller.component(Controller.COMP_DIRTREE));
        menu.add(item = new JMenuItem(Resources.MENU_CF_TOCOLLECT));
	item.addActionListener((PhotoCollectionPanel)controller.component(Controller.COMP_COLLECTION));
        menu.addSeparator();
        menu.add(item = new JMenuItem(Resources.MENU_PROPERTIES));
        item.addActionListener((JTreeTable)controller.component(Controller.COMP_DIRTREE));
        menu.addSeparator();
        menu.add(item = new JMenuItem(Resources.MENU_EXIT));
        item.addActionListener(controller);
        menubar.add(menu = new JMenu(Resources.MENU_TOOLS));
	menu.add(Controller.createTransformMenu((PhotoCollectionPanel)controller.component(Controller.COMP_COLLECTION)));
        menu.addSeparator();
        menu.add(item = new JMenuItem(Resources.MENU_EXTRACTMARKERS));
	item.addActionListener((PhotoCollectionPanel)controller.component(Controller.COMP_COLLECTION));
        menu.add(item = new JMenuItem(Resources.MENU_EXTRACTTUMBNAILS));
	item.addActionListener((PhotoCollectionPanel)controller.component(Controller.COMP_COLLECTION));
	menu.add(item = new JMenuItem(Resources.MENU_GENERATEHTML));
	item.addActionListener((PhotoCollectionPanel)controller.component(Controller.COMP_COLLECTION));
	menu.add(item = new JMenuItem(Resources.MENU_VIEW_HTML));
        item.addActionListener(controller);
        menu.addSeparator();
	menu.add(item = new JMenuItem(Resources.MENU_OPTIONS));
        item.addActionListener(controller);
        menubar.add(menu = new JMenu(Resources.MENU_HELP));
        menu.add(item = new JMenuItem(Resources.MENU_CONTENTS));
        item.addActionListener(controller);
        menu.add(item = new JMenuItem(Resources.MENU_ABOUT));
        item.addActionListener(controller);
	return menubar;
    }

    JToolBar createToolBar() {
	JToolBar result = new JToolBar();
	    JButton btn;
	    result.add(btn = new JButton(new ImageIcon(ClassLoader.getSystemResource("photoorganizer.jpg"))));
	    btn.setToolTipText(Resources.MENU_ADDTOCOLLECT);
	    btn.addActionListener((JTreeTable)controller.component(Controller.COMP_DIRTREE));
	    result.add(btn = new JButton(new ImageIcon(ClassLoader.getSystemResource("changedrive.jpg"))));
	    btn.setToolTipText(Resources.MENU_DRIVE_SEL);
	    btn.addActionListener((JTreeTable)controller.component(Controller.COMP_DIRTREE));
	    return result;
    }

    class FlashWindow extends JWindow {
	public FlashWindow() {
	    getContentPane().add(new JLabel(new ImageIcon(ClassLoader.getSystemResource("logo.jpg"))));
	    pack();
	    Dimension d = getToolkit().getScreenSize();
	    Dimension d2 = getPreferredSize();
	    setLocation((d.width-d2.width)/2, (d.height - d2.height)/2);
	    setVisible(true);
	}
    }
    
    class ROptionsMenu extends JPopupMenu {
        public ROptionsMenu() {
            JMenuItem item;
            add(item = new JMenuItem(Resources.MENU_EXTRACTTUMBNAILS));
            item.addActionListener((PhotoCollectionPanel)controller.component(Controller.COMP_COLLECTION));
            add(item = new JMenuItem(Resources.MENU_GENERATEHTML));
            item.addActionListener((PhotoCollectionPanel)controller.component(Controller.COMP_COLLECTION));
            add(item = new JMenuItem(Resources.MENU_VIEW_HTML));
            item.addActionListener(controller);
            addSeparator();
            add(item = new JMenuItem(Resources.MENU_OPTIONS));
            item.addActionListener(controller);
        }
    }

    private Controller controller;
}
