/* PhotoOrganizer 
 * Copyright (C) 1999 Dmitry Rogatkin.  All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 */
package photoorganizer.formats;
import java.io.File;
import java.util.Date;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.IOException;
import java.awt.Image;

import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.text.DecimalFormat;

public abstract class AbstractImageInfo extends BasicIo {
    final static DateFormat dateformat = new SimpleDateFormat("yyyy:MM:dd HH:mm:ss");
    final static DecimalFormat fnumberformat = new DecimalFormat("F1:#0.0#");
    
    public AbstractImageInfo(InputStream is, byte[] data, int offset, String name) {
	this.is = is;
	this.data = data;
	this.offset = offset;
	this.name = name;
	readInfo();
    }

    public String getName() {
	return name;
    }

    public void setName(String name) {
	this.name = name;
    }

    public abstract void readInfo();

    public abstract String getFormat();
    
    public abstract String getMake();

    public abstract String getModel();
    
    public abstract String getDataTimeOriginalString();

    public abstract float getFNumber();

    public abstract Rational getShutter();

    public abstract boolean isFlash();

    public abstract String getQuality();

    public abstract float getFocalLength();

    public abstract String getReport();

    public abstract Image getThumbnailImage(BasicJpeg im);

    public void saveThumbnailImage(BasicJpeg im, OutputStream os) throws IOException {
        if (os != null)
            os.close();
    }

    public void save(OutputStream os) throws IOException {
        if (os != null && data != null) {
            os.write(data);
            os.close();
        }
    }

    public Date getDateTimeOriginal() {
	try {
	    return dateformat.parse(getDataTimeOriginalString());
	} catch (NullPointerException e) {
	} catch (ParseException e) {
	    System.err.println(""+e);
	}
	return null;
    }

    InputStream is;
    int offset;
    String name;
}