/* AlbumSelectionPanel 
 * Copyright (C) 1999 Dmitry Rogatkin.  All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 */
package photoorganizer.renderer;

import javax.swing.*;
import java.awt.event.*;

import photoorganizer.*;
import photoorganizer.album.*;

public class AlbumOptionsTab extends JPanel implements ActionListener, Persistentable {
    public final static String SECNAME = "AlbumOptions";
    public final static String DRIVER = "JDBCDriver";
    public final static String DBURL = "DatabaseURL";
    public final static String USER = "User";
    public final static String PSWD = "Password";
    public final static String IGNORE_DB_ERR = "IgnoreDatabaseErrors";

    public final static String DEFAULT_JDBC_DRIVER = "com.ms.jdbc.odbc.JdbcOdbcDriver";
    public final static String DEFAULT_DB_URL = "jdbc:odbc:PhotoOrganizer";

    public AlbumOptionsTab(Controller controller) {
        this.controller = controller;
        setLayout(new GranulatedLayout(8, 7, Resources.CTRL_VERT_SIZE, Resources.CTRL_VERT_GAP, Resources.CTRL_HORIS_INSET, Resources.CTRL_HORIZ_GAP));
        add(new JLabel(Resources.LABEL_JDBCDRV_CLASS), "0,0,3");
        add(tf_driver_class = new JTextField(), "0,1,5");
        add(new JLabel(Resources.LABEL_DATABASE_URL), "0,2,2");
        add(tf_db_url = new JTextField(), "0,3,5");
        add(new JLabel(Resources.LABEL_USER), "5,0,1");
        add(tf_db_user = new JTextField(), "5,1,2");
        add(new JLabel(Resources.LABEL_PASSWORD), "5,2,2");
        add(tf_db_pswd = new JPasswordField(), "5,3,2");
	add(cb_nowarn = new JCheckBox(Resources.LABEL_NOJDBCWARN), "0,4,3");
        JButton btn;
        add(btn = new JButton(Resources.CMD_CREATEDB), "0,5,2");
        btn.addActionListener(this);
        add(btn = new JButton(Resources.CMD_INITJDBC), "2,5,2");
        btn.addActionListener(this);
    }
    
    public void actionPerformed(ActionEvent a) {
        String cmd = a.getActionCommand();
        Access access = ((AlbumPane)controller.component(Controller.COMP_ALBUMPANEL)).access;
        if (Resources.CMD_CREATEDB.equals(cmd)) {
            if (JOptionPane.showConfirmDialog(this,
                Resources.LABEL_COFIRMDROPTABLE, Resources.TITLE_COFIRMATION,
                JOptionPane.OK_CANCEL_OPTION) == JOptionPane.OK_OPTION) {
                access.forgetTables();
                access.createTables();
                ((AlbumPane)controller.component(Controller.COMP_ALBUMPANEL)).invalidateTree();
            }
        } else if (Resources.CMD_INITJDBC.equals(cmd)) {
            access.init();
        }
    }           
    
    public void load() {
        Serializer s = controller.getSerializer();
        String p = (String)s.getProperty(SECNAME, DRIVER);
        if (p!=null && p.length() >0)
            tf_driver_class.setText(p);
        else
            tf_driver_class.setText(DEFAULT_JDBC_DRIVER);
        p = (String)s.getProperty(SECNAME, DBURL);
        if (p!=null && p.length() >0)
            tf_db_url.setText(p);
        else
            tf_db_url.setText(DEFAULT_DB_URL);
        p = (String)s.getProperty(SECNAME, USER);
        if (p!=null && p.length() >0)
            tf_db_user.setText(p);
        p = (String)s.getProperty(SECNAME, PSWD);
        if (p!=null)
            tf_db_pswd.setText(p);
        cb_nowarn.setSelected(Serializer.getInt(s.getProperty(SECNAME, IGNORE_DB_ERR), 0) == 1);
    }
    
    public void save() {
        Serializer s = controller.getSerializer();
        s.setProperty(SECNAME, DRIVER, tf_driver_class.getText());
        s.setProperty(SECNAME, DBURL, tf_db_url.getText());
        s.setProperty(SECNAME, USER, tf_db_user.getText());
        s.setProperty(SECNAME, PSWD, new String(tf_db_pswd.getPassword()));
        s.setProperty(SECNAME, IGNORE_DB_ERR, cb_nowarn.isSelected()?Resources.I_YES:Resources.I_NO);
    }
    
    Controller controller;
    JTextField tf_driver_class, tf_db_url, tf_db_user;
    JPasswordField tf_db_pswd;
    JCheckBox cb_nowarn;
}