/* AlbumPane.java
 * Copyright (C) 1999 Dmitry Rogatkin.  All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 */
package photoorganizer.renderer;

import javax.swing.*;
import javax.swing.tree.TreePath;
import javax.swing.tree.TreeSelectionModel;
import javax.swing.event.MouseInputAdapter;
import javax.swing.event.TreeSelectionListener;
import javax.swing.event.TreeSelectionEvent;
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.io.*;

import photoorganizer.*;
import photoorganizer.formats.*;
import photoorganizer.album.*;

public class AlbumPane extends JTree implements ActionListener, TreeSelectionListener {
    AlbumThumbnailsPanel thumbnails;
    PhotoCollectionPanel collectionpanel;

    public AlbumPane(Controller controller) {
        this.controller = controller;
        access = new Access(controller);
        setModel(new AlbumModel(access));
        getSelectionModel().setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);
        albumselpane = new AlbumSelectionPanel(controller, getModel());
	collectionpanel = (PhotoCollectionPanel)controller.component(Controller.COMP_COLLECTION);
        addTreeSelectionListener(this);
        addMouseListener(new MouseInputAdapter() {
            public void mouseClicked(MouseEvent e) {
                if ((e.getModifiers() & InputEvent.BUTTON3_MASK) > 0) {
                    Point p = ((JViewport)getParent()).getViewPosition();
                    new AlbumRightBtnMenu(AlbumPane.this).
                    show(getParent(), e.getX()-p.x, e.getY()-p.y);
                }
            }
        });
    }

    void setTumbnailsPanel(AlbumThumbnailsPanel thumbnails) {
        this.thumbnails = thumbnails;
    }

    public void actionPerformed(ActionEvent a) {
        String cmd = a.getActionCommand();
        if (cmd.equals(Resources.CMD_ADDTOALBUM)) {
	    TreePath[] paths = albumselpane.getSelectedAlbums();
	    for (int i = 0; paths != null && i < paths.length; i++)
		access.insertPicturesToAlbum(access.getAlbumId(paths[i]), albumselectiondlg.getFiles());
	    albumselectiondlg.setVisible(false);
        } else if (cmd.equals(Resources.CMD_CANCEL)) {
	    albumselectiondlg.setVisible(false);
        } else if (cmd.equals(Resources.MENU_ADDTOCOLLECT)) {
	    collectionpanel.add(access.getAlbumContents(access.getAlbumId(getSelectionPath())));
        } else if (cmd.equals(Resources.MENU_DELETE)) {
	    access.deleteAlbum(access.getAlbumId(getSelectionPath()));
            invalidateTree();
        } else if (cmd.equals(Resources.MENU_DELETE_COMPLETLY)) {
	    access.deleteAlbum(access.getAlbumId(getSelectionPath()));
	    invalidateTree();
        } else if (cmd.equals(Resources.MENU_RENAME)) {
        } else if (cmd.equals(Resources.MENU_PROPERTIES)) {
	}
    }

    public void valueChanged(TreeSelectionEvent e) {
        thumbnails.updateImages(access.getAlbumContents(access.getAlbumId(e.getPath())));
    }

    public void addToAlbum(File[] files) {
        if (files.length == 0)
            return;
        if (albumselectiondlg == null) {
            albumselectiondlg = new AlbumSlectionDialog(files);
        }
	else
	    albumselectiondlg.files = files;
        albumselectiondlg.setTitle(Resources.TITLE_SELECT_ALBUM+files[0].getAbsolutePath()+(files.length > 1?",...":""));
        albumselectiondlg.setVisible(true);
    }

    void deletePicture(String name) {
        access.deletePicture(access.getAlbumId(getSelectionPath()), name);
    }

    public void invalidateTree() {
        ((AlbumModel)getModel()).
            fireTreeStructureChanged(this, new Object[]{getModel().getRoot()}, null, null);
    }

    class AlbumSlectionDialog extends JDialog {
	public AlbumSlectionDialog(File[] files) {
	    super(controller.getMainFrame(), true);
	    this.files = files;
	    getContentPane().add(albumselpane, "Center");
	    JPanel buttons = new JPanel();
	    JButton bt;
	    buttons.setLayout(new FlowLayout(FlowLayout.RIGHT));
	    buttons.add(bt = new JButton(Resources.CMD_ADDTOALBUM));
	    bt.addActionListener(AlbumPane.this);
	    buttons.add(bt = new JButton(Resources.CMD_CANCEL));
	    bt.addActionListener(AlbumPane.this);
	    getContentPane().add(buttons, "South");
            pack();
	}

	File[] getFiles() {
	    return files;
	}
	File[] files;
    }

    class AlbumRightBtnMenu extends JPopupMenu {
        AlbumRightBtnMenu(ActionListener listener) {
            JMenuItem item;
            add(item = new JMenuItem(Resources.MENU_ADDTOCOLLECT));
            item.addActionListener(listener);
	    addSeparator();
            add(item = new JMenuItem(Resources.MENU_DELETE));
            item.addActionListener(listener);
            add(item = new JMenuItem(Resources.MENU_DELETE_COMPLETLY));
            item.addActionListener(listener);
            add(item = new JMenuItem(Resources.MENU_RENAME));
            item.addActionListener(listener);
            addSeparator();
            add(item = new JMenuItem(Resources.MENU_PROPERTIES));
            item.addActionListener(listener);
        }
    }

    private Controller controller;
    private AlbumSlectionDialog albumselectiondlg;
    private AlbumSelectionPanel albumselpane;
    Access access;
}