/* AlbumThumbnailsPanel.java
 * Copyright (C) 1999 Dmitry Rogatkin.  All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 */
package photoorganizer.renderer;

import javax.swing.*;
import javax.swing.event.MouseInputAdapter;
import java.awt.*;
import java.awt.event.*;
import java.io.File;
import java.io.InputStream;
import java.io.IOException;

import photoorganizer.formats.*;
import photoorganizer.*;

public class AlbumThumbnailsPanel extends ThumbnailsPanel {

    PhotoImagePanel imagepanel;
    AlbumPane       album;
    PhotoCollectionPanel collectionpanel;

    public AlbumThumbnailsPanel(Controller controller) {
        super(controller);
        imagepanel = (PhotoImagePanel)controller.component(Controller.COMP_IMAGEALBUMPANEL);
        album = (AlbumPane)controller.component(Controller.COMP_ALBUMPANEL);
	collectionpanel = (PhotoCollectionPanel)controller.component(Controller.COMP_COLLECTION);
        album.setTumbnailsPanel(this);
    }

    JPopupMenu getRightButtonMenu(ActionListener listener) {
        return new AlbumThumbsRightBtnMenu(listener);
    }

    void doSpecificAction(BasicJpeg format, ActionEvent a, Thumbnail source) {
        String cmd = a.getActionCommand();
        int op;
        if (cmd.equals(Resources.MENU_SHOW)) {
            imagepanel.updateImage(format);
        } else if (cmd.equals(Resources.MENU_DELETE)) {
            album.deletePicture(format.getLocationName());
            remove((Component)source);
	    adjustDimension();
        } else if (cmd.equals(Resources.MENU_ADDTOCOLLECT)) {
	    collectionpanel.add(new File(format.getLocationName()));
        } else if (cmd.equals(Resources.MENU_PRINT)) {
            controller.print(new File[]{new File(format.getLocationName())});
        }
    }

    class AlbumThumbsRightBtnMenu extends JPopupMenu {
        AlbumThumbsRightBtnMenu(ActionListener listener) {
            JMenuItem item;
            add(item = new JMenuItem(Resources.MENU_SHOW));
            item.addActionListener(listener);
            add(item = new JMenuItem(Resources.MENU_ADDTOCOLLECT));
            item.addActionListener(listener);
            addSeparator();
            add(item = new JMenuItem(Resources.MENU_DELETE));
            item.addActionListener(listener);
            addSeparator();
            add(item = new JMenuItem(Resources.MENU_PROPERTIES));
            item.addActionListener(listener);
            addSeparator();
            add(item = new JMenuItem(Resources.MENU_PRINT));
            item.addActionListener(listener);
        }
    }
}