/* PhotoOrganizer 
 * Copyright (C) 1999 Dmitry Rogatkin.  All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 */
package photoorganizer.renderer;

import javax.swing.*;
import javax.swing.event.MouseInputAdapter;
import java.awt.*;
import java.awt.event.*;
import java.io.File;
import java.io.InputStream;
import java.io.IOException;

import photoorganizer.formats.*;
import photoorganizer.*;

public class CollectionThumbnailsPanel extends ThumbnailsPanel {
    
    PhotoImagePanel imagepanel;
    PhotoCollectionPanel collectionpanel;
    public CollectionThumbnailsPanel(Controller controller) {
        super(controller);
        imagepanel = (PhotoImagePanel)controller.component(Controller.COMP_IMAGECOLLCTPANEL);
    }
    
    void setCollection(PhotoCollectionPanel collectionpanel) {
        this.collectionpanel = collectionpanel;
    }

    JPopupMenu getRightButtonMenu(ActionListener listener) {
        return new CollectThumbsRightBtnMenu(listener);
    }

    void doSpecificAction(BasicJpeg format, ActionEvent a, Thumbnail source) {
        String cmd = a.getActionCommand();
        int op;
        if (cmd.equals(Resources.MENU_SHOW)) {
            imagepanel.updateImage(format);
        } else if (cmd.equals(Resources.MENU_DELETE)) {
            collectionpanel.delete(new int[]{collectionpanel.findIndexOf(source)});
        } else if (cmd.equals(Resources.MENU_RENAME)) {
            collectionpanel.rename(new int[]{collectionpanel.findIndexOf(source)});
        } else if (cmd.equals(Resources.MENU_SELECT)) {
            int s = collectionpanel.findIndexOf(source);
            if (s>=0)
                collectionpanel.addRowSelectionInterval(s, s);
        } else if (cmd.equals(Resources.MENU_PRINT)) {
            controller.print(new File[]{new File(format.getLocationName())});
        } else if (cmd.equals(Resources.MENU_ADDTOALBUM)) {
            ((AlbumPane)controller.component(Controller.COMP_ALBUMPANEL))
                .addToAlbum(new File[]{ new File(format.getLocationName()) });
        } else if ((op = Controller.convertCmdToTrnasformOp(cmd)) != -1) {
            collectionpanel.transform(new int[]{collectionpanel.findIndexOf(source)}, op);
        }
    }
    
    class CollectThumbsRightBtnMenu extends JPopupMenu {
        CollectThumbsRightBtnMenu(ActionListener listener) {
            JMenuItem item;
            add(item = new JMenuItem(Resources.MENU_SELECT));
            item.addActionListener(listener);
            add(item = new JMenuItem(Resources.MENU_SHOW));
            item.addActionListener(listener);
            addSeparator();
            add(item = new JMenuItem(Resources.MENU_ADDTOALBUM));
            item.addActionListener(listener);
            add(item = new JMenuItem(Resources.MENU_RENAME));
            item.addActionListener(listener);
            add(item = new JMenuItem(Resources.MENU_DELETE));
            item.addActionListener(listener);
            addSeparator();
            add(item = new JMenuItem(Resources.MENU_PRINT));
            item.addActionListener(listener);
            addSeparator();
            this.add(Controller.createTransformMenu(listener));
            addSeparator();
            add(item = new JMenuItem(Resources.MENU_PROPERTIES));
            item.addActionListener(listener); // setPopupSize(this.getPreferredSize());
        }
    }
}