/* PhotoOrganizer 
 * Copyright (C) 1999 Dmitry Rogatkin.  All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 */
package photoorganizer.renderer;

import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;

import photoorganizer.*;
import photoorganizer.formats.FileNameFormat;
import photoorganizer.directory.JDirectoryChooser;

public class MiscellaneousOptionsTab extends JPanel implements ActionListener, Persistentable {
    public final static String SECNAME = "PhotoCollectionOptions";
    public final static String DATEFORMAT = "DateFormat";
    public final static String TIMEFORMAT = "TimeFormat";
    public final static String TOOLBAR = "ToolBar";
    public final static String STATUSBAR = "StatusBar";
    public final static String MENUBAR = "MenuBar";
    public final static String TABPOS = "TabPosition";
    public final static String FC_FOLDER = "FlashCardDirectory";
    public final static String SHOWWARNDLG = "ShowWarnDlg";
    // LIST_ONTOP, LIST_ONBOTTOM, LIST_ONLEFT, LIST_ONRIGHT
    final static Integer[] TABPOSTABLE = {new Integer(SwingConstants.TOP),
	new Integer(SwingConstants.BOTTOM), new Integer(SwingConstants.LEFT), new Integer(SwingConstants.RIGHT) };

    final static String formathelp = 
     "G        era designator          (Text)              AD\n"+
     "y        year                    (Number)            1996\n"+
     "M        month in year           (Text & Number)     July & 07\n"+
     "d        day in month            (Number)            10\n"+
     "h        hour in am/pm (1~12)    (Number)            12\n"+
     "H        hour in day (0~23)      (Number)            0\n"+
     "m        minute in hour          (Number)            30\n"+
     "s        second in minute        (Number)            55\n"+
     "S        millisecond             (Number)            978\n"+
     "E        day in week             (Text)              Tuesday\n"+
     "D        day in year             (Number)            189\n"+
     "F        day of week in month    (Number)            2 (2nd Wed in July)\n"+
     "w        week in year            (Number)            27\n"+
     "W        week in month           (Number)            2\n"+
     "a        am/pm marker            (Text)              PM\n"+
     "k        hour in day (1~24)      (Number)            24\n"+
     "K        hour in am/pm (0~11)    (Number)            0\n"+
     "z        time zone               (Text)              Pacific Standard Time\n"+
     "'        escape for text         (Delimiter)\n"+
     "''       single quote            (Literal)           '";

    public MiscellaneousOptionsTab(Controller controller) {
	this.controller = controller;
        setLayout(new GranulatedLayout(8, 6, 18, 2, 8, 8));
        JPanel canvas = new JPanel();
        canvas.setBorder(new TitledBorder(new EtchedBorder(), Resources.TITLE_SHOW));
        canvas.setOpaque(false);
        add(canvas, "0,0,8,4,16,8");
	add(cb_menu = new JCheckBox(Resources.LABEL_MENU), "1,1,2,1");
        add(cb_tool = new JCheckBox(Resources.LABEL_TOOLBAR), "3,1,3,1");
        add(cb_status = new JCheckBox(Resources.LABEL_SATUSBAR), "1,2,2,1");
        add(new JLabel(Resources.LABEL_TABPOS), "3,2,2,1");
        add(mb_positions = new JComboBox(Resources.POSITIONS), "5,2,2,1");
        add(new JLabel(Resources.LABEL_DATEFMT), "0,4,2,1,20");
        add(tf_date = new JTextField(), "2,4,1,1");
        add(new JLabel(Resources.LABEL_TIMEFMT), "3,4,2,1,20");
        add(tf_time = new JTextField(), "5,4,1,1");
        JButton button;
        add(button = new JButton(Resources.CMD_HELP), "6,4,2,1,12");
        button.addActionListener(this);

	add(new JLabel(Resources.LABEL_CF_DIR), "0,6,2");
	add(tf_fc_folder = new JTextField(), "2,6,4");
	add(button = new JButton(Resources.CMD_BROWSE), "6,6,2");
        button.addActionListener(this);
	add(cb_warn = new JCheckBox(Resources.LABEL_SHOWWARNDLG), "0,7,3,1");
    }
    
    public void actionPerformed(ActionEvent a) {
        if (a.getActionCommand() == Resources.CMD_HELP) {
            JOptionPane.showMessageDialog(this, formathelp, Resources.TITLE_HELP, JOptionPane.PLAIN_MESSAGE);
        } if (a.getActionCommand() == Resources.CMD_BROWSE) {
            JDirectoryChooser dc = new JDirectoryChooser(new JFrame(), tf_fc_folder.getText(), null);
            if (dc.getDirectory() != null)
                tf_fc_folder.setText(dc.getDirectory());
	}
    }

    public void load() {
        Serializer s = controller.getSerializer();
	tf_date.setText(s.arrayToString(s.getProperty(SECNAME, DATEFORMAT)));
	tf_time.setText(s.arrayToString(s.getProperty(SECNAME, TIMEFORMAT)));
        tf_fc_folder.setText(s.arrayToString(s.getProperty(SECNAME, FC_FOLDER)));
	cb_menu.setSelected(Serializer.getInt(s.getProperty(SECNAME, MENUBAR), 1) == 1);
	cb_status.setSelected(Serializer.getInt(s.getProperty(SECNAME, STATUSBAR), 1) == 1);
	cb_tool.setSelected(Serializer.getInt(s.getProperty(SECNAME, TOOLBAR), 0) == 1);
	cb_warn.setSelected(Serializer.getInt(s.getProperty(SECNAME, SHOWWARNDLG), 0) == 1);
	switch(Serializer.getInt(s.getProperty(SECNAME, TABPOS), SwingConstants.BOTTOM)) {
	case SwingConstants.TOP: 
	    mb_positions.setSelectedIndex(0);
	    break;
	case SwingConstants.BOTTOM: 
	    mb_positions.setSelectedIndex(1);
	    break;
	case SwingConstants.LEFT: 
	    mb_positions.setSelectedIndex(2);
	    break;
	case SwingConstants.RIGHT: 
	    mb_positions.setSelectedIndex(3);
	    break;
	}
    }
    
    public void save() {
        Serializer s = controller.getSerializer();
        FileNameFormat.datemask = tf_date.getText().trim();
        s.setProperty(SECNAME, DATEFORMAT, FileNameFormat.datemask);
        FileNameFormat.timemask = tf_time.getText().trim();
        s.setProperty(SECNAME, TIMEFORMAT, FileNameFormat.timemask);
        s.setProperty(SECNAME, FC_FOLDER, tf_fc_folder.getText());
        
	s.setProperty(SECNAME, MENUBAR, cb_menu.isSelected()?Resources.I_YES:Resources.I_NO);
	s.setProperty(SECNAME, STATUSBAR, cb_status.isSelected()?Resources.I_YES:Resources.I_NO);
	s.setProperty(SECNAME, TOOLBAR, cb_tool.isSelected()?Resources.I_YES:Resources.I_NO);
	s.setProperty(SECNAME, SHOWWARNDLG, cb_warn.isSelected()?Resources.I_YES:Resources.I_NO);
	s.setProperty(SECNAME, TABPOS, TABPOSTABLE[mb_positions.getSelectedIndex()]);
    }

    Controller controller;
    JTextField tf_date, tf_time, tf_fc_folder;
    JCheckBox  cb_menu, cb_status, cb_tool, cb_warn;
    JComboBox mb_positions;
}