/* PhotoOrganizer 
 * Copyright (C) 1999 Dmitry Rogatkin.  All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 */
package photoorganizer.renderer;

import javax.swing.*;
import javax.swing.event.MouseInputAdapter;
import javax.swing.event.TableModelEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.event.ListSelectionEvent;
import javax.swing.table.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.io.*;

import photoorganizer.*;
import photoorganizer.formats.*;

public class PhotoCollectionPanel extends JTable implements ActionListener, Persistentable {
    final static String SECNAME = "PhotoCollectionPanel";
    final static String COLWIDTH = "ColumnWidthes";
    
    private Controller controller;
    private Point lastmouse;
    private CollectionThumbnailsPanel thumbnailspanel;
    private StatusBar statusbar;
    private Class fclass;
    private boolean keeporigmarkers, transformmove, transformaddsel;
    public final static String DEFTNMASK = "tumbnail%00c.jpg";

    public PhotoCollectionPanel(Controller controller) {
	setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
        this.controller = controller;
        thumbnailspanel = (CollectionThumbnailsPanel)controller.component(Controller.COMP_THUMBCOLLCTPANEL);
        thumbnailspanel.setCollection(this);
        statusbar = (StatusBar)controller.component(Controller.COMP_STATUSBAR);
	addMouseListener(new MouseInputAdapter() {
	    public void mouseClicked(MouseEvent e) {
		int m = e.getModifiers();
		lastmouse = new Point(e.getX(), e.getY());
		if ((m & InputEvent.BUTTON3_MASK) > 0)
		    new RButtonMenu().show(PhotoCollectionPanel.this, e.getX(), e.getY());
	    }
	});
	setModel(new CollectionModel());
        setMinimumSize(Resources.MIN_PANEL_DIMENSION);
	getSelectionModel().addListSelectionListener(new ListSelectionListener() {
            public void valueChanged(ListSelectionEvent e) {
		int f = e.getFirstIndex();
		int l = e.getLastIndex();
		if (f > l) {
		    l = e.getFirstIndex();
		    f = e.getLastIndex();
		}
		CollectionModel model = (CollectionModel) getModel();
                Thumbnail tn;
		for (int i = f; i <= l; i++) 
		    if ((tn = model.getElementAt(i)) != null)
                        tn.select(isRowSelected(i));
            }
        });
    }
    
    public void actionPerformed(ActionEvent a) {
        String cmd = a.getActionCommand();
	CollectionModel model = (CollectionModel) getModel();
	int [] selections = getSelectedRows();

        if (cmd.equals(Resources.MENU_SELECT)) {
	    int sel = rowAtPoint(lastmouse);
	    if (sel >= 0)
		addRowSelectionInterval(sel, sel);
        } else if (cmd.equals(Resources.MENU_SELECTALL)) {
	    selectAll();
        } else if (cmd.equals(Resources.MENU_CF_TOCOLLECT)) {
            String path = (String)controller.getSerializer().getProperty(MiscellaneousOptionsTab.SECNAME, MiscellaneousOptionsTab.FC_FOLDER);
            if (path != null && path.length() > 0)
                add(new File(path));
        } else if (cmd.equals(Resources.MENU_EXTRACTMARKERS)) {
            extractMarkers(selections);
        } else if (cmd.equals(Resources.MENU_ADDTOALBUM)) {
            File[] files = new File[selections.length];
            for (int i = 0; i < selections.length; i++)
                files[i] = new File(model.getElementAt(selections[i]).getImageInfo().getLocationName());
            ((AlbumPane)controller.component(Controller.COMP_ALBUMPANEL))
            .addToAlbum(files);
        } else if (cmd.equals(Resources.MENU_RENAME)) {
            rename(selections);
        } else if (cmd.equals(Resources.MENU_DELETE)) {
            delete(selections);
	} else if (cmd.equals(Resources.MENU_SHOW)) {
	    int sel = rowAtPoint(lastmouse);
	    if (sel >= 0) {
		BasicJpeg format = model.getElementAt(sel).getImageInfo();
		if (format != null)
		    ((PhotoImagePanel)controller.component(Controller.COMP_IMAGECOLLCTPANEL)).updateImage(format);
	    }
	} else if (cmd.equals(Resources.MENU_PROPERTIES)) {
            for(int i=0; i<selections.length; i++) {
		PropertiesPanel.showProperties(model.getElementAt(selections[i]).getImageInfo(), controller);
            }
        } else if (cmd.equals(Resources.MENU_EXTRACTTUMBNAILS)) {
	    BasicJpeg format;
	    Serializer s = controller.getSerializer();
	    String destpath = (String)s.getProperty(WebPublishOptionsTab.SECNAME, WebPublishOptionsTab.WEBROOT);
	    if (destpath == null || destpath.length() == 0)
		destpath = controller.getSerializer().getHomeDirectory();
	    String m = (String)s.getProperty(ThumbnailsOptionsTab.SECNAME, ThumbnailsOptionsTab.FILEMASK);
	    if (m == null || m.length() == 0)
		m = DEFTNMASK;
	    FileNameFormat fnf = new FileNameFormat(m);
            for(int i=0; i<selections.length; i++) {
		format = model.getElementAt(selections[i]).getImageInfo();
		AbstractImageInfo ii = format.getImageInfo();
		try {
		    if (ii != null)
			ii.saveThumbnailImage(format,
			new FileOutputStream(new File(destpath,FileNameFormat.makeValidPathName(fnf.format(format)))));
		} catch(IOException ioe) {
		    System.err.println(""+ioe);
		}
            }
        } else if (cmd.equals(Resources.MENU_GENERATEHTML)) {
            if (selections.length == 0)
                return;
            String htmlname = (String)JOptionPane.showInputDialog(this,
                Resources.LABEL_HTML_NAME, Resources.TITLE_CREATEHTML,
                JOptionPane.QUESTION_MESSAGE, null, null, "");
	    if (htmlname != null) {
		if (htmlname.indexOf('.') < 0)
		    htmlname += Resources.EXT_HTML;
		try {
		    // prepare array of files
		    File [] files = new File[selections.length];
		    for(int i=0; i<selections.length; i++)
			files[i] = new File(model.getElementAt(selections[i]).getImageInfo().getLocationName());
		    new HtmlProducer(controller).produce(htmlname, files);
		} catch (IOException e) {
		    System.err.println("Exception in HTML producing "+e);
		}
	    }
        } else if (cmd.equals(Resources.MENU_PRINT)) {
            print(selections);
        }
        int op;
	if ((op = Controller.convertCmdToTrnasformOp(cmd)) != -1) {
            transform(selections, op);
        }
    }
    
    public int add(File f) {
	CollectionModel model = (CollectionModel) getModel();
	int end, start = model.getRowCount();
	end = start;
	if (f.isFile()) {
	    thumbnailspanel.addImage(f);
	    end++;
	} else if (f.isDirectory()) {
	    String[] files = f.list();
	    String path = f.getPath();
	    for (int i = 0; i<files.length; i++) {
		f = new File(path, files[i]);
		if (f.isFile()) {
		    thumbnailspanel.addImage(f);
		    end++;
		}
	    }
	}
	model.fireTableRowsInserted(start, end);
	return end;
    }

    public int add(File[] fs) {
	CollectionModel model = (CollectionModel) getModel();
	int end, start = model.getRowCount();
	end = start;
	for (int i=0; i<fs.length; i++) {
	    if (fs[i].isFile()) {
		thumbnailspanel.addImage(fs[i]);
		end++;
	    } else if (fs[i].isDirectory()) {
		String[] files = fs[i].list();
		String path = fs[i].getPath();
		for (int k = 0; k<files.length; k++) {
		    fs[i] = new File(path, files[k]);
		    if (fs[i].isFile()) {
			thumbnailspanel.addImage(fs[i]);
			end++;
		    }
		}
	    }
	}
	model.fireTableRowsInserted(start, end);
	return end;
    }

    String transformName(File file, int op) {
        Serializer s = controller.getSerializer();
        String destpath = (String)s.getProperty(RenameOptionsTab.SECNAME, RenameOptionsTab.DESTFOLDER);
        if (destpath == null || destpath.length() == 0)
            destpath = controller.getSerializer().getHomeDirectory();
	String m = (String)s.getProperty(TransformOptionsTab.SECNAME, TransformOptionsTab.MASK);
        if (m != null) {
            return destpath+File.separatorChar+
		FileNameFormat.makeValidPathName(new FileNameFormat(m, op).format(new BasicJpeg(file)));
        } else
            return destpath+File.separatorChar+file.getName();
    }
    
    int findIndexOf(Thumbnail im) {
        CollectionModel model = (CollectionModel) getModel();
        for (int i=0; i<model.getRowCount(); i++) {
            if (model.getElementAt(i) == im)
                return i;
        }
        return -1;
    }

    void print(int [] selections) {
        File []files = new File[selections.length];
        CollectionModel model = (CollectionModel) getModel();
        for (int i=0; i<selections.length; i++)
            files[i] = new File(model.getElementAt(selections[i]).getImageInfo().getLocationName());
        controller.print(files);
    }

    void extractMarkers(int [] selections) {
        Serializer s = controller.getSerializer();
        CollectionModel model = (CollectionModel) getModel();
        String  name;
        BasicJpeg im;
        AbstractImageInfo ii;
        int dp;
        for(int i=0; i<selections.length; i++) {
            im = model.getElementAt(selections[i]).getImageInfo();
	    ii = im.getImageInfo();
            if (ii != null) {
                name = ii.getName();
                dp = name.lastIndexOf('.');
                if (dp > 0)
                    name = name.substring(0, dp+1);
                else
                    name += '.';
                name += ii.getFormat();
                try {
                    ii.save(new FileOutputStream(new File(im.getParentPath(), name)));
                } catch(IOException e) {
                    System.err.println("Exception in markers extraction "+e);
                }
            }
        }
    }

    void rename(int [] selections) {
        Serializer s = controller.getSerializer();
        String destpath = (String)s.getProperty(RenameOptionsTab.SECNAME, RenameOptionsTab.DESTFOLDER);
        if (destpath == null || destpath.length() == 0)
            destpath = controller.getSerializer().getHomeDirectory();
        final Object value;
        if (Serializer.getInt(s.getProperty(RenameOptionsTab.SECNAME, RenameOptionsTab.ASKEDIT), 0) == 1) {
            value = JOptionPane.showInputDialog(this,
                Resources.LABEL_NEW_NAME, Resources.TITLE_RENAME,
                JOptionPane.QUESTION_MESSAGE, null, null, 
                s.getProperty(RenameOptionsTab.SECNAME, RenameOptionsTab.MASK));
        } else
            value = s.getProperty(RenameOptionsTab.SECNAME, RenameOptionsTab.MASK);
        if (value == null)
            return;
        final int []sels =selections;
        final String dp = destpath;
        statusbar.displayInfo(Resources.INFO_RENAMING);
        statusbar.setProgress(sels.length);
	final boolean removeafter = s.getInt(s.getProperty(RenameOptionsTab.SECNAME, RenameOptionsTab.REMOVEAFTER), 0) == 1;
        
        new Thread(new Runnable () {
            public void run() {
                BasicJpeg format;
                int minr=0, maxr=0;
                CollectionModel model = (CollectionModel) getModel();
                for(int i=0; i<sels.length; i++) {
                    format = model.getElementAt(sels[i]).getImageInfo();
                    File dest = new File(dp, FileNameFormat.makeValidPathName(
                        new FileNameFormat((String)value).format(format)));
                    if (format.renameTo(dest)) {
                        if (sels[i] > maxr)
                            maxr = sels[i];
                        if (sels[i] < minr)
                            minr = sels[i];
                    }
		    if (removeafter) {
			model.getElementAt(sels[i]).select(false);
			model.markDelete(sels[i]);
		    }
                    statusbar.tickProgress();
                }
		if (removeafter) {
		    model.removeAllMarked();
		    model.fireTableRowsDeleted(minr, maxr);
		} else
		    model.fireTableRowsUpdated(minr, maxr);
                statusbar.clearInfo();
                statusbar.clearProgress();
            }
        }).start();
    }
    
    void delete(int [] selections) {
        CollectionModel model = (CollectionModel) getModel();
        statusbar.displayInfo(Resources.INFO_REMOVING);
        int minr=0, maxr=0;
        for(int i=0; i<selections.length; i++) {
            model.getElementAt(selections[i]).select(false);
	    model.markDelete(selections[i]);
            if (selections[i] > maxr)
                maxr = selections[i];
            if (selections[i] < minr)
                minr = selections[i];
        }
	model.removeAllMarked();
        model.fireTableRowsDeleted(minr, maxr);
        tableChanged(new TableModelEvent(model, minr, maxr, TableModelEvent.ALL_COLUMNS, TableModelEvent.DELETE));
        statusbar.clearInfo();
    }

    void transform(int [] ss, int o) {
	class TransformWorker implements Runnable {
	    TransformWorker(int [] selections, int op) {
		this.selections = selections;
		this.op = op;
	    }
	    public void run() {
		CollectionModel model = (CollectionModel) getModel();
		loadTransformOptions();
		int minr=0, maxr=0, bound = model.getRowCount();
                String transformname;
		File file;
		for(int i=0; i<selections.length; i++) {
		    file = new File(model.getElementAt(selections[i]).getImageInfo().getLocationName());
                    transformname = transformName(file, op);
		    model.getElementAt(selections[i]).getImageInfo().
		        transform(transformname, op, keeporigmarkers, fclass);
		    if (transformmove) {
			if (file.delete()) {
			    model.removeElementAt(selections[i]);
			    if (selections[i] > maxr)
				maxr = selections[i];
			    if (selections[i] < minr)
				minr = selections[i];
			}
		    }
		    if (transformaddsel) {
			add(new File(transformname));
		    }
                    statusbar.tickProgress();
		}
		if (transformaddsel) {
                    model.fireTableRowsInserted(bound-1, bound+selections.length-1);
		    tableChanged(new TableModelEvent(model, bound-1, bound+selections.length-1, TableModelEvent.ALL_COLUMNS, TableModelEvent.INSERT));
                } 
                if (transformmove) {
		    model.fireTableRowsDeleted(minr, maxr);
		    tableChanged(new TableModelEvent(model, minr, maxr, TableModelEvent.ALL_COLUMNS, TableModelEvent.DELETE));
		}
                statusbar.clearInfo();
                statusbar.clearProgress();
	    }
	    int [] selections;
	    int op;
	}
        statusbar.displayInfo(Resources.INFO_TRANSFORMING);
        statusbar.setProgress(ss.length);
	SwingUtilities.invokeLater(new TransformWorker(ss, o));
    }

    void loadTransformOptions() {
	Serializer s = controller.getSerializer(); 
	keeporigmarkers = Serializer.getInt(s.getProperty(TransformOptionsTab.SECNAME, TransformOptionsTab.KEEP), 0) == 1;
	transformmove = Serializer.getInt(s.getProperty(TransformOptionsTab.SECNAME, TransformOptionsTab.MOVE), 0) == 0;
        transformaddsel = Serializer.getInt(s.getProperty(TransformOptionsTab.SECNAME, TransformOptionsTab.TRANSFTOSEL), 1) == 1;
	String format = (String)s.getProperty(TransformOptionsTab.SECNAME, TransformOptionsTab.FORMAT);
	fclass = null;
	if (format != null && !keeporigmarkers) {
	    if (Resources.LIST_EXIF.equals(format))
		fclass = Exif.class;
	    else if (Resources.LIST_CIFF.equals(format))
		fclass = CIFF.class;
	    else if (Resources.LIST_JFIF.equals(format))
		fclass = JFXX.class;
	    else if (Resources.LIST_EXTRN.equals(format))
		fclass = AbstractImageInfo.class;
	}
    }
    public void save() {
       	Serializer s = controller.getSerializer();
	Integer[] w = new Integer[getColumnCount()];
        for (int i = 0; i < w.length; i++) {
            w[i] = new Integer(getColumn(getColumnName(i)).getWidth());
        }
	s.setProperty(SECNAME, COLWIDTH, w);
    }
    
    public void load() {
       	Serializer s = controller.getSerializer();
	Object[] w = (Object[])s.getProperty(SECNAME, COLWIDTH);
        if (w == null)
            return;
        for (int i = 0; i < w.length; i++) {
	    getColumn(getColumnName(i)).setWidth(((Integer)w[i]).intValue());
            getColumn(getColumnName(i)).setPreferredWidth(((Integer)w[i]).intValue());
        }
    }

    class RButtonMenu extends JPopupMenu {
	RButtonMenu() {
	    JMenuItem item;
	    this.add(item = new JMenuItem(Resources.MENU_SELECT));
	    item.addActionListener(PhotoCollectionPanel.this);
	    this.add(item = new JMenuItem(Resources.MENU_SELECTALL));
	    item.addActionListener(PhotoCollectionPanel.this);
	    this.add(item = new JMenuItem(Resources.MENU_SHOW));
	    item.addActionListener(PhotoCollectionPanel.this);
            this.add(item = new JMenuItem(Resources.MENU_ADDTOALBUM));
	    item.addActionListener(PhotoCollectionPanel.this);
	    this.add(item = new JMenuItem(Resources.MENU_EXTRACTMARKERS));
	    item.addActionListener(PhotoCollectionPanel.this);
	    addSeparator();
	    this.add(item = new JMenuItem(Resources.MENU_RENAME));
	    item.addActionListener(PhotoCollectionPanel.this);
	    this.add(item = new JMenuItem(Resources.MENU_DELETE));
	    item.addActionListener(PhotoCollectionPanel.this);
	    addSeparator();
            this.add(item = new JMenuItem(Resources.MENU_PRINT));
            item.addActionListener(PhotoCollectionPanel.this);
            addSeparator();
	    this.add(Controller.createTransformMenu(PhotoCollectionPanel.this));
	    addSeparator();
	    this.add(item = new JMenuItem(Resources.MENU_PROPERTIES));
	    item.addActionListener(PhotoCollectionPanel.this);
	}
    }

    class CollectionModel extends AbstractTableModel {
	Vector collection;
	String [] colnames = {Resources.HDR_NAME, Resources.HDR_TAKEN,
	    Resources.HDR_SHUTTER, Resources.HDR_APERTURE, Resources.HDR_FLASH};
	
	public int getRowCount() {
	    return thumbnailspanel.getComponentCount();
	}
	
	public int getColumnCount() {
	    return colnames.length; 
	}
	
	public String getColumnName(int column) {
	    return colnames[column];
	}
	
	public Object getValueAt(int row, int column) {
	    BasicJpeg im = ((Thumbnail)thumbnailspanel.getComponent(row)).getImageInfo();
	    if (im == null)
		return null;
	    AbstractImageInfo ii = im.getImageInfo();
	    try {
		switch(column) {
		case 0:
		    if (ii == null)
			return im.getLocationName();
		    return ii.getName();
		case 1:
		    if (ii != null) {
			Date date = ii.getDateTimeOriginal();
			if (date != null)
			    return date;
		    }
		    break;
		case 2:
		    if (ii != null)
			return ii.getShutter();
		    break;
		case 3:
		    if (ii != null)
			return new Float(ii.getFNumber());
		    break;
		case 4:
		    if (ii != null)
			return ii.isFlash()?Boolean.TRUE:Boolean.FALSE;
		    break;
		}
	    }
	    catch  (NullPointerException ne) { }
	    
	    return null; 
	}
	
	Thumbnail getElementAt(int index) {
            try {
	    return (Thumbnail)thumbnailspanel.getComponent(index);
            } catch (ArrayIndexOutOfBoundsException e) {
                return null;
            }
	}
	
	void removeElementAt(int row) {
	    thumbnailspanel.remove(row);
	}

	void markDelete(int row) {
	    marked.addElement(thumbnailspanel.getComponent(row));
	}

	void removeAllMarked() {
	    for (int i=0; i<marked.size(); i++)
		thumbnailspanel.remove((Component)marked.elementAt(i));
	    marked.removeAllElements();
	    thumbnailspanel.adjustDimension();
	}
	
	public void fireTableRowsUpdated(int firstRow, int lastRow) {
	    super.fireTableRowsUpdated(firstRow, lastRow);
	    for (int i=firstRow; i<=lastRow; i++) {
		((Thumbnail)thumbnailspanel.getComponent(i)).update();
	    }
	}

	Vector marked = new Vector(10);
    }
}