/* PhotoOrganizer 
 * Copyright (C) 1999 Dmitry Rogatkin.  All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 */
package photoorganizer.renderer;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.io.File;
import java.awt.Toolkit;
import javax.swing.event.MouseInputAdapter;

import photoorganizer.formats.*;
import photoorganizer.*;

interface ImageChangeListener {
    public void updateImage(Image image);
}

public class PhotoImagePanel extends JPanel implements ActionListener, ImageChangeListener {
    private JScrollPane scroller;
    private ImageChangeListener  iviewer;
    private Controller controller;
    private BasicJpeg jpeg;

    public PhotoImagePanel(Controller controller) {
	this.controller = controller;
        setLayout(new BorderLayout());
        add(scroller=new JScrollPane(new ImageComponent()), "Center");
        setMinimumSize(Resources.MIN_PANEL_DIMENSION);
    }
    
    public void updateImage(Image image) {
	if (iviewer == null || iviewer == this)
            ((ImageChangeListener)this.scroller.getViewport().getView()).updateImage(image);
        else {
	    if (iviewer instanceof ImageFrame)
		((ImageFrame)iviewer).setTitle(PhotoOrganizer.PROGRAMNAME+" "+PhotoOrganizer.VERSION+" - "+jpeg.getLocationName());
	    iviewer.updateImage(image);
	}
    }

    public void updateImage(BasicJpeg jpeg) {
	this.jpeg = jpeg;
	SwingUtilities.invokeLater(new Runnable() {
	    public void run() {
		updateImage(PhotoImagePanel.this.jpeg.getImage());
	    }
	});
    }

    public void actionPerformed(ActionEvent a) {
        String cmd = a.getActionCommand();
        if (cmd.equals(Resources.MENU_POPUPWIN)) {
            if (iviewer != null) {
		if (iviewer instanceof ImageFrame)
		    return;
		if (iviewer instanceof ImageWindow)
		    ((Window)iviewer).dispose();
	    }
	    iviewer = new ImageFrame(PhotoOrganizer.PROGRAMNAME+" "+PhotoOrganizer.VERSION+" - "+jpeg.getLocationName());
        } else if (cmd.equals(Resources.MENU_INTWIN)) {
            if (iviewer != null && iviewer != this) {
                ((Window)iviewer).dispose();
                iviewer = null;
            }
        } else if (cmd.equals(Resources.MENU_FULLSCREEN)) {
            if (iviewer != null) {
		if (iviewer instanceof ImageWindow)
		    return;
		if (iviewer instanceof ImageFrame)
		    ((Window)iviewer).dispose();
	    }
	    iviewer = new ImageWindow();
        }
    }

    void trigger() {
	if (iviewer != null && iviewer != this) {
	    ((Window)iviewer).dispose();
	    iviewer = null;
	} else
	    iviewer = new ImageFrame(PhotoOrganizer.PROGRAMNAME+" "+PhotoOrganizer.VERSION+" - "+jpeg.getLocationName());
    }

    class ImageComponent extends JLabel implements ImageChangeListener {
        public ImageComponent() {
            super(new ImageIcon());
            addMouseListener(new MouseInputAdapter() {
                public void mouseClicked(MouseEvent e) {
		    int m = e.getModifiers();
		    if (e.getClickCount() == 2 && (m & InputEvent.BUTTON1_MASK) > 0) {
			trigger();
		    } else if ((m & InputEvent.BUTTON3_MASK) > 0) {
                        Point p = ((JViewport)getParent()).getViewPosition();
                        new ImageRButtonMenu(PhotoImagePanel.this).show(getParent(), e.getX()-p.x, e.getY()-p.y);
                    }
                }
            });
        }
        
        public void updateImage(Image image) {
            if (image == null)
                return;
	    ((ImageIcon)getIcon()).setImage(image);
	    revalidate();
	    repaint();
	    Serializer s = controller.getSerializer();
	    String o = s.arrayToString(s.getProperty(ThumbnailsOptionsTab.SECNAME, ThumbnailsOptionsTab.TOOLTIPMASK));
	    if (o != null) 
		setToolTipText(new FileNameFormat(o).format(jpeg));
        }

        public void paint(Graphics g) {
            try {
                super.paint(g);
            } catch (NullPointerException e) {
                // work around Sun's VM bug
            }
        }

        class ImageRButtonMenu extends JPopupMenu {
            ImageRButtonMenu(ActionListener listener) {
                JMenuItem item;
                this.add(item = new JMenuItem(Resources.MENU_POPUPWIN));
                item.addActionListener(listener);
                this.add(item = new JMenuItem(Resources.MENU_INTWIN));
                item.addActionListener(listener);
                this.add(item = new JMenuItem(Resources.MENU_FULLSCREEN));
                item.addActionListener(listener);
            }
        }
    }

    class ImageFrame extends JFrame implements ImageChangeListener {
        public ImageFrame(String caption) {
            super(caption);
            getContentPane().add(scroller);
            setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
            pack();
            setVisible(true);
	    setIconImage(controller.getMainIcon());
        }
        
        public void dispose() {
            iviewer = PhotoImagePanel.this;
            PhotoImagePanel.this.add(scroller);
            PhotoImagePanel.this.revalidate();
            super.dispose();
        }

        public void updateImage(Image image) {
            ((ImageChangeListener)scroller.getViewport().getView()).updateImage(image);
        }
    }

    class ImageWindow extends JWindow implements ImageChangeListener {
	public ImageWindow() {
            getContentPane().add(scroller);
	    pack();
            setVisible(true);
	    setSize(Toolkit.getDefaultToolkit().getScreenSize());
	}

        public void updateImage(Image image) {
            ((ImageChangeListener)scroller.getViewport().getView()).updateImage(image);
	}
	
        public void dispose() {
            iviewer = PhotoImagePanel.this;
            PhotoImagePanel.this.add(scroller);
            PhotoImagePanel.this.revalidate();
            super.dispose();
        }
    }
}


