/* PhotoOrganizer 
 * Copyright (C) 1999 Dmitry Rogatkin.  All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 */
package photoorganizer.renderer;

import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.DocumentListener;
import javax.swing.event.DocumentEvent;
import javax.swing.text.Document;
import photoorganizer.*;
import photoorganizer.formats.FileNameFormat;

public class ThumbnailsOptionsTab extends JPanel implements ActionListener, DocumentListener, Persistentable {
    public final static String SECNAME = "ThumbnailsOptions";
    final static String FIXAXIS = "FixedNumber";
    final static String VERTAXIS = "Vertical";
    final static String CELLWIDTH = "CellWidth";
    final static String CELLHEIGHT = "CellHeight";
    public final static String TOOLTIPMASK = "ToolTipMask";
    public final static String LABELMASK = "LabelMask";
    public final static String FILEMASK = "FileMask";
    
    public ThumbnailsOptionsTab(Controller controller) {
	this.controller = controller;
        setLayout(new GranulatedLayout(6, 6, Resources.CTRL_VERT_SIZE, Resources.CTRL_VERT_GAP, Resources.CTRL_HORIS_INSET, Resources.CTRL_HORIZ_GAP));
	add(new JLabel(Resources.LABEL_NUM_COL_OR_ROW), "0,0,3,1");
	add(tf_fixedaxis = new JTextField(), "3,0,1,1");
	tf_fixedaxis.setHorizontalAlignment(JTextField.RIGHT);
	add(cb_scrollv = new JCheckBox(Resources.LABEL_SCROLL_VERT), "4,0,2,1,12");
	add(new JLabel(Resources.LABEL_THUMBNAILFILE_MASK), "0,2,2,1");
	add(tf_thumbfilemask = new JTextField(), "2,2,3,1");
        tf_thumbfilemask.getDocument().addDocumentListener(this);
        add(button1 = new JButton(Resources.LABEL_INS_R), "5,2,1,1");
        button1.addActionListener(this);
	add(new JLabel(Resources.LABEL_THUMBNAILSIZE), "0,1,2,1");
	add(new JLabel(Resources.LABEL_WIDTH_SHORT, SwingConstants.RIGHT), "2,1,1,1");
	add(tf_cellwidth = new JTextField(SwingConstants.RIGHT), "3,1,1,1");
	tf_cellwidth.setHorizontalAlignment(JTextField.RIGHT);
	add(new JLabel(Resources.LABEL_HEIGHT_SHORT, SwingConstants.RIGHT), "4,1,1,1");
	add(tf_cellheight = new JTextField(SwingConstants.RIGHT), "5,1,1,1");
	tf_cellheight.setHorizontalAlignment(JTextField.RIGHT);
	add(new JLabel(Resources.LABEL_TTIP_MASK), "0,3,2,1");
	add(tf_tooltipmask = new JTextField(), "2,3,3,1");
        tf_tooltipmask.getDocument().addDocumentListener(this);
        add(button = new JButton(Resources.LABEL_INS_R), "5,3,1,1");
        button.addActionListener(this);
	add(new JLabel(Resources.LABEL_LABEL_MASK), "0,4,2,1");
	add(tf_labelmask = new JTextField(), "2,4,3,1");
        tf_labelmask.getDocument().addDocumentListener(this);
        add(button = new JButton(Resources.LABEL_INS_R), "5,4,1,1");
        button.addActionListener(this);
        add(mb_format = new JComboBox(Resources.MASKS[1]), "4,5,2,1");
	mb_format.setLightWeightPopupEnabled(false);
    }
    
    public void actionPerformed(ActionEvent a) {
        int sel = mb_format.getSelectedIndex();
        if (sel >= 0) {
	    if (a.getSource() == button)
		tf_labelmask.replaceSelection(Resources.MASKS[0][sel]);
	    else if (a.getSource() == button1)
		tf_thumbfilemask.replaceSelection(Resources.MASKS[0][sel]);
	    else
		tf_tooltipmask.replaceSelection(Resources.MASKS[0][sel]);
        }
    }

    public void insertUpdate(DocumentEvent e) {
        changedUpdate(e);
    }
    
    public void removeUpdate(DocumentEvent e) {
        changedUpdate(e);
    }
    
    public void changedUpdate(DocumentEvent e) {
        Document d = e.getDocument();
        if (d == tf_tooltipmask.getDocument())
            tf_tooltipmask.setToolTipText(new FileNameFormat(tf_tooltipmask.getText()).format(controller.sampleJpeg));
        else if (d == tf_labelmask.getDocument())
            tf_labelmask.setToolTipText(new FileNameFormat(tf_labelmask.getText()).format(controller.sampleJpeg));
        else if (d == tf_thumbfilemask.getDocument())
            tf_thumbfilemask.setToolTipText(FileNameFormat.makeValidPathName(new FileNameFormat(tf_thumbfilemask.getText()).format(controller.sampleJpeg)));

        FileNameFormat.counter--;
    }

    public void load() {
        Serializer s = controller.getSerializer();
	Integer i = (Integer)s.getProperty(SECNAME, FIXAXIS);
	if (i != null)
	    tf_fixedaxis.setText(i.toString());
	i = (Integer)s.getProperty(SECNAME, VERTAXIS);
	if (i != null)
	    cb_scrollv.setSelected(i.intValue() == 1);
	i = (Integer)s.getProperty(SECNAME, CELLWIDTH);
	if (i != null)
	    tf_cellwidth.setText(i.toString());
	i = (Integer)s.getProperty(SECNAME, CELLHEIGHT);
	if (i != null)
	    tf_cellheight.setText(i.toString());
	tf_tooltipmask.setText(s.arrayToString(s.getProperty(SECNAME, TOOLTIPMASK)));
	tf_labelmask.setText(s.arrayToString(s.getProperty(SECNAME, LABELMASK)));
	tf_thumbfilemask.setText(s.arrayToString(s.getProperty(SECNAME, FILEMASK)));
    }
    
    public void save() {
        Serializer s = controller.getSerializer();
	try {
	    s.setProperty(SECNAME, FIXAXIS, new Integer(tf_fixedaxis.getText()));
	} catch(NumberFormatException e) {
	}
	s.setProperty(SECNAME, VERTAXIS, cb_scrollv.isSelected()?Resources.I_YES:Resources.I_NO);
	try {
	    s.setProperty(SECNAME, CELLWIDTH, new Integer(tf_cellwidth.getText()));
	} catch(NumberFormatException e) {
	}
	try {
	    s.setProperty(SECNAME, CELLHEIGHT, new Integer(tf_cellheight.getText()));
	} catch(NumberFormatException e) {
	}
	s.setProperty(SECNAME, TOOLTIPMASK, tf_tooltipmask.getText());
	s.setProperty(SECNAME, LABELMASK, tf_labelmask.getText());
	s.setProperty(SECNAME, FILEMASK, tf_thumbfilemask.getText());
    }

    private JTextField tf_cellwidth, tf_cellheight,
	tf_fixedaxis, tf_tooltipmask, tf_labelmask, tf_thumbfilemask;
    private JCheckBox cb_scrollv;
    private JButton button, button1;
    private JComboBox mb_format;
    private Controller controller;
}