/* PhotoOrganizer 
 * Copyright (C) 1999 Dmitry Rogatkin.  All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 */
package photoorganizer.renderer;

import javax.swing.*;
import javax.swing.event.MouseInputAdapter;
import javax.swing.border.*;
import java.awt.*;
import java.awt.event.*;
import java.io.File;
import java.io.InputStream;
import java.io.IOException;

import photoorganizer.formats.*;
import photoorganizer.*;

public class ThumbnailsPanel extends JPanel {
    static final int THUMB_SIZE_X = 162;
    static final int THUMB_SIZE_Y = 142; // should be 960*THUMB_SIZE_X/1280+FONT_HEIGHT
    static final int THUMBS_IN_ROW = 5;
    
    public ThumbnailsPanel(Controller controller) {
        this.controller = controller;
	collectionpanel = (PhotoCollectionPanel)controller.component(Controller.COMP_COLLECTION);
	albumpanel      = (AlbumPane)controller.component(Controller.COMP_ALBUMPANEL);
        setMinimumSize(Resources.MIN_PANEL_DIMENSION);
    }

    void calculateLayout() {
	Serializer s = controller.getSerializer();
	Integer i = (Integer)s.getProperty(ThumbnailsOptionsTab.SECNAME, ThumbnailsOptionsTab.VERTAXIS);
	v=THUMBS_IN_ROW;
	h=0;
	if (i != null && i.intValue() == 0) {
	    i = (Integer)s.getProperty(ThumbnailsOptionsTab.SECNAME, ThumbnailsOptionsTab.FIXAXIS);
	    if (i!= null) {
		v=0; h=i.intValue();
	    }
	} else {
	    i = (Integer)s.getProperty(ThumbnailsOptionsTab.SECNAME, ThumbnailsOptionsTab.FIXAXIS);
	    if (i!= null) {
		h=0; v=i.intValue();
	    }
	}
        sx = THUMB_SIZE_X;
        sy = THUMB_SIZE_Y;
        i = (Integer)s.getProperty(ThumbnailsOptionsTab.SECNAME, ThumbnailsOptionsTab.CELLWIDTH);
        if (i != null)
            sx = i.intValue();
        else
            s.setProperty(ThumbnailsOptionsTab.SECNAME, ThumbnailsOptionsTab.CELLWIDTH, new Integer(sx));
        i = (Integer)s.getProperty(ThumbnailsOptionsTab.SECNAME, ThumbnailsOptionsTab.CELLHEIGHT);
        if (i != null)
            sy = i.intValue();
        else
            s.setProperty(ThumbnailsOptionsTab.SECNAME, ThumbnailsOptionsTab.CELLHEIGHT, new Integer(sy));
        setLayout(new GridLayout(h,v));
    }
    
    public Dimension getPreferredSize() {
        return preferredsize;
    }
    
    public void updateImages(File file) {
        updateImages(new File[]{file});
    }

    public void updateImages(File []files) {
        removeAll();
        calculateLayout();
        BasicJpeg format;
        for(int i = 0; i < files.length; i++) {
            if (files[i].isFile()) {
                format = new BasicJpeg(files[i]);
                if (format.isValid())
                    add(new ThumbnailComponent(format));
            } else if (files[i].isDirectory()) {
                String []names = files[i].list();
                String path = files[i].getPath();
                for(int n = 0; n < names.length; n++) {
                    format = new BasicJpeg(new File(path, names[n]));
                    if (format.isValid())
                    {
                        add(new ThumbnailComponent(format));
                    }
                }
            }
        }
        adjustDimension();
    }

    public void addImage(File file) {
	calculateLayout();
        BasicJpeg format = new BasicJpeg(file);
	if (format.isValid())
	    add(new ThumbnailComponent(format));
        adjustDimension();
    }
    
    void adjustDimension() {
	if (v!=0)
	    preferredsize =
		new Dimension(v*sx, (getComponentCount()/v+1)*sy);
	else
	    preferredsize =
		new Dimension((getComponentCount()/h+1)*sx, h*sy);
        setSize(preferredsize);
	revalidate();
        repaint();
    }

    JPopupMenu getRightButtonMenu(ActionListener listener) {
	return new FastMenu(listener);
    }

    void doSpecificAction(BasicJpeg format, ActionEvent a, Thumbnail source) {
	String cmd = a.getActionCommand();
	if (cmd.equals(Resources.MENU_ADDTOCOLLECT)) {
	    collectionpanel.add(new File(format.getLocationName()));
	} else if (cmd.equals(Resources.MENU_ADDTOALBUM)) {
	    albumpanel.addToAlbum(new File[]{new File(format.getLocationName())});
	} else if (cmd.equals(Resources.MENU_SHOW)) {
	    controller.fireDisplayImage(format);
	} else if (cmd.equals(Resources.MENU_RENAME)) {
            Serializer s = controller.getSerializer();
	    boolean success = false;
            if (format.isValid() && Serializer.getInt(s.getProperty(RenameOptionsTab.SECNAME, RenameOptionsTab.ASKEDIT), 0) == 0) {
                success = format.renameTo(new File(format.getParentPath(), FileNameFormat.makeValidPathName(RenameOptionsTab.getNewName(format, controller))));
            } else {
		String value;
                if (format.isValid())
                    value = format.getParentPath()+File.separatorChar+
                    FileNameFormat.makeValidPathName(RenameOptionsTab.getNewName(format, controller));
		else
		    value = format.getLocationName();
                value = (String)JOptionPane.showInputDialog(this,
                    Resources.LABEL_NEW_NAME, Resources.TITLE_RENAME,
                    JOptionPane.QUESTION_MESSAGE, null, null, value);
                if (value != null)
                    success = format.renameTo(new File(value.toString()));
            }
	    if (success)
		source.update();
	} else if (cmd.equals(Resources.MENU_DELETE)) {
	    Serializer s = controller.getSerializer();
	    if (Serializer.getInt(s.getProperty(MiscellaneousOptionsTab.SECNAME, MiscellaneousOptionsTab.SHOWWARNDLG), 0) == 1 &&
		JOptionPane.showConfirmDialog(this, Resources.LABEL_COFIRMDELETE+format.getLocationName(),
		Resources.TITLE_COFIRMATION, JOptionPane.OK_CANCEL_OPTION) != JOptionPane.OK_OPTION)
		return;
            File of = new File(format.getLocationName());
	    if (of.delete()) {
		remove((Component)source);
		adjustDimension();
	    }
	} else if (cmd.equals(Resources.MENU_PRINT)) {
            controller.print(new File[]{new File(format.getLocationName())});
        }
    }

    class ThumbnailComponent extends JLabel implements ActionListener, Thumbnail {
        Dimension preferredsize = new Dimension(ThumbnailsPanel.this.sx, ThumbnailsPanel.this.sy);
        private BasicJpeg format;
        
        public ThumbnailComponent() {
            super(new ImageIcon());
            setHorizontalTextPosition(SwingConstants.CENTER);
            setVerticalTextPosition(SwingConstants.BOTTOM);
            addMouseListener(new MouseInputAdapter() {
		public void mouseClicked(MouseEvent e) {
		    int m = e.getModifiers();
		    if (e.getClickCount() == 2 && (m & InputEvent.BUTTON1_MASK) > 0) {
			// show full image
                        doSpecificAction(format, new ActionEvent(this, 0, Resources.MENU_SHOW), ThumbnailComponent.this);
		    } else if ((m & InputEvent.BUTTON3_MASK) > 0)
			getRightButtonMenu(ThumbnailComponent.this).show(ThumbnailComponent.this, e.getX(), e.getY());
		}
	    }); 
        }
        
        public Dimension getPreferredSize() {
            return preferredsize;
        }
	
	public void actionPerformed(ActionEvent a) {
	    String cmd = a.getActionCommand();
	    if (cmd.equals(Resources.MENU_PROPERTIES)) {
		PropertiesPanel.showProperties(format, controller);
	    } else 
		doSpecificAction(format, a, this);
	}
        
        public ThumbnailComponent(BasicJpeg format) {
            this();
            updateImage(format);
        }
        
        void updateImage(BasicJpeg format) {
            this.format = format;
	    Serializer s = controller.getSerializer();
	    String o = s.arrayToString(s.getProperty(ThumbnailsOptionsTab.SECNAME, ThumbnailsOptionsTab.TOOLTIPMASK));
            Image image = null;
            if (format.getImageInfo() != null)
                image = format.getImageInfo().getThumbnailImage(format);
	    if (o != null) {
		setToolTipText(new FileNameFormat(o).format(format));
	    } else
		setToolTipText(format.getImageInfo().getReport());
            if (image != null) {
                ((ImageIcon)getIcon()).setImage(image);
		o = (String)s.getProperty(ThumbnailsOptionsTab.SECNAME, ThumbnailsOptionsTab.LABELMASK);
		if (o != null) {
		    setText(new FileNameFormat(o).format(format));
		} else
                    setText(""+format.getImageInfo().getDateTimeOriginal());
                return;
            }
            if (format.getImageInfo() != null)
                setText(Resources.LABEL_NOTUMBNAIL+format.getImageInfo().getMake());
            else
                setText(Resources.LABEL_NOTUMBNAIL+format.getLocationName());
        }

        public void paint(Graphics g) {
            try {
                super.paint(g);
            } catch (NullPointerException e) {
                // work around Sun's VM bug
            }
        }

	public BasicJpeg getImageInfo() {
	    return format;
	}
	
	public void select(boolean on) {
	    setBorder(on?new LineBorder(Color.blue):null);
	}

	public void update() {
	    updateImage(format);
	}
    }

    Dimension preferredsize = new Dimension(0, 160);
    Controller controller;
    private PhotoCollectionPanel collectionpanel;
    private AlbumPane albumpanel;
    private int v,h;
    private int sx, sy; // cell size
}

