/* WebPublishOptionsTab 
 * Copyright (C) 1999 Dmitry Rogatkin.  All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 */
package photoorganizer.renderer;

import javax.swing.*;
import javax.swing.filechooser.FileSystemView;
import java.awt.event.*;
import java.io.IOException;

import photoorganizer.*;
import photoorganizer.ftp.*;
import photoorganizer.directory.JDirectoryChooser;

public class WebPublishOptionsTab extends JPanel implements ActionListener, Persistentable {
    public final static String SECNAME   = "WebPublishOptions";
    public final static String WEBROOT   = "WebRoot";
    public final static String HOSTNAME  = "HostName";
    public final static String USER      = "UserName";
    public final static String PASSWORD  = "Password";
    public final static String TIMEOUT   = "ConnTimeout";
    public final static String PROXYHOST = "ProxyHost";
    public final static String PROXYPORT = "ProxyPort";
    public final static String USEPROXY  = "UseProxy";
    public final static String ASKPSWD   = "SavePassword";
    public final static String USEFTP    = "FtpRoot";
    public final static String HTMLTEMPL = "HTMLTemplate";
    final static int DEFTIMEOUT = 20;

    public WebPublishOptionsTab(Controller controller) {
        this.controller = controller;
        setLayout(new GranulatedLayout(8, 7, Resources.CTRL_VERT_SIZE, Resources.CTRL_VERT_GAP, Resources.CTRL_HORIS_INSET, Resources.CTRL_HORIZ_GAP));
        add(new JLabel(Resources.LABEL_HTML_TEMPLATE), "0,0,2");
        add(tf_htmltmpl = new JTextField(), "2,0,4");
        add(bt_brws_htmltmpl = new JButton(Resources.CMD_BROWSE), "6,0,2");
	bt_brws_htmltmpl.addActionListener(this);
        add(new JLabel(Resources.LABEL_WEB_ROOT), "0,1,2");
        add(tf_webroot = new JTextField(), "2,1,4");
        add(bt_brws_wroot = new JButton(Resources.CMD_BROWSE), "6,1,2");
	bt_brws_wroot.addActionListener(this);
        add(cb_useftp = new JCheckBox(Resources.LABEL_USE_FTP), "0,2,2");
        add(new JLabel(Resources.LABEL_FTP_HOST), "2,2,2");
        add(tf_hostname = new JTextField(), "4,2,4");
        add(new JLabel(Resources.LABEL_LOGIN), "0,3,2");
        add(tf_user = new JTextField(), "2,3,2");
        add(new JLabel(Resources.LABEL_PASSWORD), "4,3,2");
        add(tf_pswd = new JPasswordField(), "6,3,2");
        add(cb_keeppswd = new JCheckBox(Resources.LABEL_SAVE_PSWD), "4,4,3");
        add(new JLabel(Resources.LABEL_CONN_TIMEOUT), "0,4,3");
        add(tf_conntimeout = new JTextField(), "3,4,1");
        tf_conntimeout.setHorizontalAlignment(JTextField.RIGHT);
        add(cb_useproxy = new JCheckBox(Resources.LABEL_USE_PROXY), "0,5,2");
        add(new JLabel(Resources.LABEL_HOST), "0,6,2");
        add(tf_proxyhost = new JTextField(), "2,6,2");
        add(new JLabel(Resources.LABEL_PORT), "4,6,2");
        add(tf_proxyport = new JTextField(), "6,6,2");
        tf_proxyport.setHorizontalAlignment(JTextField.RIGHT);
    }

    public void actionPerformed(ActionEvent a) {
        if (a.getSource() == bt_brws_wroot) {
            FileSystemView fv = null;
            if (cb_useftp.isSelected()) {
                Ftp ftp = new Ftp(getConnectionInfo(controller.getSerializer()));
		if (!ftp.isConnected) {
		    JOptionPane.showMessageDialog(controller.getMainFrame(), Resources.LABEL_ERR_FTP_CONNECT,
			Resources.TITLE_ERROR, JOptionPane.ERROR_MESSAGE);
		    return ;
		}
                try {
                    ftp.login();
                    fv = new FtpFileSystemView(ftp);
                } catch (IOException e) {
                    JOptionPane.showMessageDialog(this, Resources.LABEL_ERR_FTP_LOGIN+e,
                        Resources.TITLE_ERROR, JOptionPane.ERROR_MESSAGE);
                    return ;
                }
            }
            JDirectoryChooser dc = new JDirectoryChooser(new JFrame(), tf_webroot.getText(), fv);
            if (dc.getDirectory() != null)
                tf_webroot.setText(dc.getDirectory());
        } else if (a.getSource() == bt_brws_htmltmpl) {
            String d = tf_htmltmpl.getText();
            if (d.length() == 0)
                d = controller.getSerializer().getHomeDirectory();
            JFileChooser fc = new JFileChooser(d);
            fc.addChoosableFileFilter(controller.getHtmlFilter());
            fc.setFileFilter(controller.getHtmlFilter());
            fc.setDialogTitle(Resources.TITLE_HTML_TEMPL);
            if (fc.showOpenDialog(this) == JFileChooser.APPROVE_OPTION)
                tf_htmltmpl.setText(fc.getSelectedFile().getAbsolutePath());
        }
    }

    public static FtpConnectionInfo getConnectionInfo(Serializer s) {
        connectioninfo.startDirectory = (String)s.getProperty(SECNAME, WEBROOT);
        connectioninfo.host = (String)s.getProperty(SECNAME, HOSTNAME);
        connectioninfo.user = (String)s.getProperty(SECNAME, USER);
        connectioninfo.password = (String)s.getProperty(SECNAME, PASSWORD);
        connectioninfo.proxyHost = (String)s.getProperty(SECNAME, PROXYHOST);
        try {
            connectioninfo.proxyPort = ((Integer)s.getProperty(SECNAME, PROXYPORT)).intValue();
        } catch(Throwable e) {
            connectioninfo.proxyPort = 0;
        }
        try {
            connectioninfo.timeout = ((Integer)s.getProperty(SECNAME, TIMEOUT)).intValue();
        } catch(Throwable e) {
            connectioninfo.timeout = DEFTIMEOUT*1000;
        }
        try {
            connectioninfo.useProxy = ((Integer)s.getProperty(SECNAME, USEPROXY)).intValue() == 1;
        } catch(Throwable e) {
            connectioninfo.useProxy = false;
        }
        try {
            connectioninfo.askPassword = ((Integer)s.getProperty(SECNAME, ASKPSWD)).intValue() == 1;
        } catch(Throwable e) {
            connectioninfo.askPassword = true;
        }
        return connectioninfo;
    }

    public void load() {
        Serializer s = controller.getSerializer();
	Integer i = (Integer)s.getProperty(SECNAME, USEFTP);
	if (i != null)
	    cb_useftp.setSelected(i.intValue() == 1);
        getConnectionInfo(s);
        if (connectioninfo.startDirectory != null)
            tf_webroot.setText(connectioninfo.startDirectory);
        if (connectioninfo.host != null)
            tf_hostname.setText(connectioninfo.host);
        if (connectioninfo.user != null)
            tf_user.setText(connectioninfo.user);
        if (connectioninfo.password != null)
            tf_pswd.setText(connectioninfo.password);
        if (connectioninfo.proxyHost != null)
            tf_proxyhost.setText(connectioninfo.proxyHost);
        tf_proxyport.setText(""+connectioninfo.proxyPort);
        tf_conntimeout.setText(""+(connectioninfo.timeout/1000));
        cb_useproxy.setSelected(connectioninfo.useProxy);
        cb_keeppswd.setSelected(connectioninfo.askPassword);
        String f = (String)s.getProperty(SECNAME, HTMLTEMPL);
        if (f != null)
            tf_htmltmpl.setText(f);
    }
    
    public void save() {
        Serializer s = controller.getSerializer();
        s.setProperty(SECNAME, WEBROOT, tf_webroot.getText());
        s.setProperty(SECNAME, HOSTNAME, tf_hostname.getText());
        s.setProperty(SECNAME, USER, tf_user.getText());
        s.setProperty(SECNAME, PASSWORD, new String(tf_pswd.getPassword()));
        s.setProperty(SECNAME, PROXYHOST, tf_proxyhost.getText());
        Integer i;
        try {
            i = new Integer(tf_proxyport.getText());
        } catch(NumberFormatException e) {
            i = Resources.I_NO;
        }
        s.setProperty(SECNAME, PROXYPORT, i);
        try {
            i = new Integer(tf_conntimeout.getText()+"000");
        } catch(NumberFormatException e) {
            i = new Integer(DEFTIMEOUT*1000);
        }
        s.setProperty(SECNAME, TIMEOUT, i);
        s.setProperty(SECNAME, USEPROXY, cb_useproxy.isSelected()?Resources.I_YES:Resources.I_NO);
        s.setProperty(SECNAME, ASKPSWD, cb_keeppswd.isSelected()?Resources.I_YES:Resources.I_NO);
        s.setProperty(SECNAME, USEFTP, cb_useftp.isSelected()?Resources.I_YES:Resources.I_NO);
        s.setProperty(SECNAME, HTMLTEMPL, tf_htmltmpl.getText());
    }
    
    Controller controller;
    JTextField tf_webroot, tf_hostname, tf_proxyhost, tf_proxyport,
        tf_user, tf_conntimeout, tf_htmltmpl;
    JPasswordField tf_pswd;
    JCheckBox cb_useproxy, cb_keeppswd, cb_useftp;
    JButton bt_brws_htmltmpl, bt_brws_wroot;
    static FtpConnectionInfo connectioninfo = new FtpConnectionInfo();
}